/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsBaCommon.h"
#include "apps/scan/SsApScIpMapMaint.h"
#include "netutils/SsNuPinger.h"
#include "SsStControl.h"

STRANGEAPPTHREAD(StrangeAppThread_IpMapperMaint)
{
	SsApScIpMapMaint *poApp = (SsApScIpMapMaint *)poStrangeApp;
	SsNuPinger oPingUtil;
	Tuint nCurCycle = 1;
	char szMessage[128];
	poApp->m_bIsRunning = true;
	TMapMachine *psCurMachine;
	Tboolean bIsValid = False;
	Tuint nCurMachine = 0;

	OutputDebugString("---- IP Mapper Maint Started -----\n");

	while(1)
	{
		if(poApp->m_bHaltCommandRequested)
		{
			sprintf(szMessage, "&R&[ * &0&IP Map Maint Service Stopped &R& * ]\n");
			poApp->vPrintMessage(szMessage);
			break;
		}

		psCurMachine = SS_CONTROL()->poGetValidMachine(nCurMachine);
		if(psCurMachine)
		{
			// Ping the current machine
			oPingUtil.bRetrieveName(psCurMachine->m_szDomain, psCurMachine->m_szIpAddress);
      if(psCurMachine->m_nScanFlags != c_nScanFlagIndexFTP)
      {
				if(oPingUtil.bPingByIP(psCurMachine->m_szIpAddress))
				{
					char szComputerName[256];

					bIsValid = True;
          psCurMachine->m_nTimeOfLastNamePing = time(NULL);
					if(oPingUtil.bGetNetbiosName(szComputerName, psCurMachine->m_szIpAddress))
					{
						strcpy(psCurMachine->m_szNetName, szComputerName);
					}
					else
					{
						strcpy(psCurMachine->m_szNetName, "");
					}
					sprintf(szMessage, "%s&b&: &0&%s (&b&%s&0&)\n", psCurMachine->m_szIpAddress, psCurMachine->m_szDomain, psCurMachine->m_szNetName);
				}
				else
				{
					// If this computer hasn't responded in a set amount of time, consider it invalid
          if(((time(NULL) - psCurMachine->m_nTimeOfLastNamePing) > g_nNameExpireSeconds) && (psCurMachine->m_szNetName[0] != 0))
          {
            sprintf(szMessage, "%s&b&: &0&%s (&g&Expiring %s&0&)\n", psCurMachine->m_szIpAddress, psCurMachine->m_szDomain, psCurMachine->m_szNetName);
						strcpy(psCurMachine->m_szNetName, "");
          }
          else
          {
					  sprintf(szMessage, "%s&b&: &0&%s (&g&Offline&0&)\n", psCurMachine->m_szIpAddress, psCurMachine->m_szDomain, psCurMachine->m_szNetName);
          }
				}
      }
      else
      {
					sprintf(szMessage, "%s&b&: &0&%s (&b&ftp site&0&)\n", psCurMachine->m_szIpAddress, psCurMachine->m_szDomain);
      }
			poApp->vPrintMessage(szMessage);
		}

		if(++nCurMachine >= SS_CONTROL()->nGetNumValidMachines())
		{
			nCurMachine = 0;
			poApp->vPrintMessage("&R&-----&0& Completed Cycle &R&-----\n\n");

			// Sleep for 30 seconds
			for(Tuint SleepCounter=0; SleepCounter<60; SleepCounter++)
			{
				if(poApp->m_bHaltCommandRequested)
				{
					break;
				}
				Sleep(500);
			}
			if(poApp->m_bHaltCommandRequested)
			{
				sprintf(szMessage, "&R&[ * &0&IP Map Maint Service Stopped &R& * ]\n");
				poApp->vPrintMessage(szMessage);
				break;
			}
			
			poApp->vResetConsole();
			sprintf(szMessage, "&R&-----&0& Beginning Cycle %d &R&-----\n", ++nCurCycle);
			poApp->vPrintMessage(szMessage);
		}
	}

  OutputDebugString("---- IP Mapper Maint Halted -----\n");

	poApp->m_bIsRunning = false;
	poApp->m_bHaltCommandRequested = false;
	_endthread();
}

void SsApScIpMapMaint::vBeginExecution()
{
	if(!m_bIsRunning)
	{
		RUN_STRANGEAPPTHREAD(StrangeAppThread_IpMapperMaint);
	}
}



//--------------------------------------------------------------------------------------------------
// $Footer: strangeapp_pinger.cpp $
//
//--------------------------------------------------------------------------------------------------


