/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "SsBaCommon.h"
#include "apps/scan/SsApScPinger.h"
#include "netutils/SsNuPinger.h"
#include "SsStControl.h"

STRANGEAPPTHREAD(StrangeAppThread_Pinger)
{
	SsApScPinger *poApp = (SsApScPinger *)poStrangeApp;
	SsNuPinger oPingUtil;
	bool bMachineUp;
	char szMessage[128];
	Tuint nCurMachine=0;
	TMachine *psCurMachine;
	TMapMachine *psValidMachine;
	Tuint nCurCycle = 1;
	poApp->m_bIsRunning = true;

	OutputDebugString("---- StrangePinger Started -----\n");

	while(1)
	{
		if(poApp->m_bHaltCommandRequested)
		{
			sprintf(szMessage, "&R&[ * &0&Ping Service Stopped &R& * ]\n");
			poApp->vPrintMessage(szMessage);
			break;
		}

		psCurMachine = SS_CONTROL()->poGetSearchMachine(nCurMachine);
		if(psCurMachine)
		{
			// Ping the current machine
			bMachineUp = oPingUtil.bPingByIP(psCurMachine->m_szIpAddress);
			psCurMachine->m_bMachineUp = bMachineUp;
			
			sprintf(szMessage, "%s&b&:&0& %s is &0&%s\n", psCurMachine->m_szIpAddress, psCurMachine->m_szName, bMachineUp ? "Online" : "&g&Offline");
			psValidMachine = SS_CONTROL()->poGetValidMachine(psCurMachine->m_nValidMachineIndex);
			psValidMachine->m_bMachineUp = bMachineUp;
			poApp->vPrintMessage(szMessage);
		}

		if(++nCurMachine >= SS_CONTROL()->nGetNumSearchMachines())
		{
			nCurMachine = 0;
			poApp->vPrintMessage("&R&-----&0& Completed Cycle &R&-----\n\n");
			// Sleep for 60 seconds
			for(Tuint SleepCounter=0; SleepCounter<120; SleepCounter++)
			{
				if(poApp->m_bHaltCommandRequested)
				{
					break;
				}
				Sleep(500);
			}
			if(poApp->m_bHaltCommandRequested)
			{
				sprintf(szMessage, "&R&[ * &0&Pinger Service Stopped &R& * ]\n");
				poApp->vPrintMessage(szMessage);
				break;
			}
			
			poApp->vResetConsole();
			sprintf(szMessage, "&R&-----&0& Beginning Cycle %d &R&-----\n", ++nCurCycle);
			poApp->vPrintMessage(szMessage);
		}

	}

  OutputDebugString("---- StrangePinger Halted -----\n");

	poApp->m_bIsRunning = false;
	poApp->m_bHaltCommandRequested = false;
	_endthread();
}

void SsApScPinger::vBeginExecution()
{
	if(!m_bIsRunning)
	{
		RUN_STRANGEAPPTHREAD(StrangeAppThread_Pinger);
	}
}

//--------------------------------------------------------------------------------------------------
// $Footer: strangeapp_pinger.cpp $
//
//--------------------------------------------------------------------------------------------------


