/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

//
// SsApBase.h
//

#ifndef _INC_SS_AP_BASE
#define _INC_SS_AP_BASE

#include "../SsBaCommon.h"

//----------------------------------------------------------------------------
//    Definitions / Enumerations / Typedefs
//----------------------------------------------------------------------------
#define STRANGEAPPTHREAD(FunctionName) void FunctionName(void *poStrangeApp)
#define RUN_STRANGEAPPTHREAD(FunctionName) _beginthread(FunctionName, 0, (void *)this)

//----------------------------------------------------------------------------
//    Class Prototypes
//----------------------------------------------------------------------------
class SsApBase
{
	protected:
		Tboolean m_bIsInitialized;
		Tboolean m_bIsRunning;
		Tboolean m_bHaltCommandRequested;

		HWND m_hConsole;

	public:
		SsApBase();
		virtual ~SsApBase();

		void vResetValues();

		virtual void vInit();
		virtual void vRelease();

		virtual void vBeginExecution() = 0;
		virtual void vRequestStop();

		virtual void vPrintMessage(char *pszMessage);
		virtual void vResetConsole();

		inline Tboolean bIsInitialized()	{ return m_bIsInitialized; }
		inline Tboolean bIsRunning()			{ return m_bIsRunning; }
		inline void vAttachToConsoleWindow(HWND hConsole)	{ m_hConsole = hConsole; }
};

#endif

