/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsBaCommon.h"
#include "apps/SsApIndexer.h"
#include "netutils/SsNuPinger.h"
#include "SsStControl.h"
#include "apps/SsApIndexer_ftp.h"

Queue::Queue()
{
	m_psLastNode = m_psFirstNode = NULL;
}

Queue::~Queue()
{
	vRelease();
}

void Queue::vEnqueue(char *data)
{
	TNode *psNewNode = new TNode;

	psNewNode->m_psNextNode = NULL;
	strncpy(psNewNode->m_szData, data, 256);
	if(m_psLastNode)
	{
		m_psLastNode->m_psNextNode = psNewNode;
		m_psLastNode = psNewNode;
	}
	else
	{
		m_psFirstNode = psNewNode;
		m_psLastNode = psNewNode;
	}
}

void Queue::vDequeue(char *pszBuffer)
{
	if(!m_psFirstNode)
	{
		pszBuffer[0] = 0;
	}
	else
	{
		if(pszBuffer)
		{
			strncpy(pszBuffer, m_psFirstNode->m_szData, 256);
		}
		TNode *psDeleteNode = m_psFirstNode;
		if(m_psLastNode == m_psFirstNode)
		{
			m_psLastNode = NULL;
			m_psFirstNode = NULL;
		}
		else
		{
			m_psFirstNode = m_psFirstNode->m_psNextNode;
		}
		delete psDeleteNode;
	}
}

void Queue::vRelease()
{
	while(!bIsEmpty())
	{
		vDequeue(NULL);		
	}
}

Tboolean bFindFtpSite(char *szIpAddress, char *szUsername, char *szPassword, char *szHostname, int &rfnPort)
{
	Tchar szFtpFilename[256];
	sprintf(szFtpFilename, "%s\\data\\FtpSites.txt", g_aszRootPath);
	FILE *pfFtpInfo = fopen(szFtpFilename, "r");
	char buffer[256], szAddress[64];

	fgets(buffer, 256, pfFtpInfo);
	int nSites = atoi(buffer);
	for(int nCurSite = 0; nCurSite < nSites; nCurSite++)
	{
		fgets(buffer, 256, pfFtpInfo);
		strcpy(szAddress, buffer);
		szAddress[strlen(szAddress)-1] = 0; // Trim newline

		if(stricmp(szIpAddress, szAddress) == 0)
		{
			fgets(buffer, 256, pfFtpInfo);	strcpy(szUsername, buffer); szUsername[strlen(szUsername)-1] = 0;
			fgets(buffer, 256, pfFtpInfo);	strcpy(szPassword, buffer); szPassword[strlen(szPassword)-1] = 0;
			fgets(buffer, 256, pfFtpInfo);	strcpy(szHostname, buffer); szHostname[strlen(szHostname)-1] = 0;
			fgets(buffer, 256, pfFtpInfo);	rfnPort = atoi(buffer);
			return 1;
		}
		fgets(buffer, 256, pfFtpInfo);	
		fgets(buffer, 256, pfFtpInfo);	
		fgets(buffer, 256, pfFtpInfo);	
		fgets(buffer, 256, pfFtpInfo);		
	}
	return 0;
}

void vStoreFile(Tchar *pszFileName, Tuint nDirIndex, Tuint nFileSize, FILE *pFile)
{
	Tchar cSep = '<';

	fwrite((char*)pszFileName, strlen(pszFileName), 1, pFile);
  fwrite((char*)&cSep, 1, 1, pFile);
  fwrite((char*)&nDirIndex, sizeof(Tuint), 1, pFile);
  fwrite((char*)&nFileSize, sizeof(Tuint), 1, pFile);
}

Tuint nStoreDir(char* DirName, FILE *pFile)
{
   char NL;
   fpos_t FP;
   fgetpos(pFile,&FP); //may need +1
   //_strupr(DirName);
   fwrite((char*)DirName,strlen(DirName),1,pFile);
   NL=0; ////
   fwrite((char*)&NL,1,1,pFile);
   return((int)FP);
}

Tuint bIndexMachine(SsApIndexer *poApp, TMapMachine *psCurMachine)
{
	Tuint nCurTime = time(NULL);
	Tchar szMessage[512];
	Queue oDirQ;

	// Ping to see if computer is alive
	SsNuPinger Pinger;
	if(!Pinger.bPingByIP(psCurMachine->m_szIpAddress))
	{
		sprintf(szMessage, "&g&\\\\%s: Machine Offline\n", psCurMachine->m_szNetName);
		poApp->vPrintMessage(szMessage);
		return 0;
	}	

	if(psCurMachine->m_nScanFlags == c_nScanFlagIndexFTP)
	{
		char szUsername[256], szPassword[256];
		char szHostname[256];
		int  nPort;

		if(bFindFtpSite(psCurMachine->m_szIpAddress, szUsername, szPassword, szHostname, nPort))
		{
			sprintf(szMessage,"&b&\\\\%s: Indexing as FTP Site\n", psCurMachine->m_szNetName);
			poApp->vPrintMessage(szMessage);
			if(bIndexFtpSite(poApp, psCurMachine->m_szNetName, szUsername, szPassword, szHostname, nPort))
			{
				psCurMachine->m_nTimeOfLastScan = nCurTime;             
				return 1;
			}
		}	
		else
		{
			sprintf(szMessage,"&R&\\\\%s: FTP Site not found\n", psCurMachine->m_szNetName);
			poApp->vPrintMessage(szMessage);
		}
		return 0;
	}

	Tchar szUserName[512];
	Tchar szPassword[512];
	Tchar szComputerName[512];
	Tchar szFilename[512];
	Tchar szDir[512];
	NETRESOURCE oNetResource[16];
	NETRESOURCE *poEnumResources;
	LPNETRESOURCE lpnr = &oNetResource[0];
	DWORD nNumResources, nEnumResourceSize;
	Tchar cEnumResourceMemory[16384];
	FILE *pDirFile=NULL, *pNameFile=NULL;
	NETRESOURCE Connection;

	poEnumResources = (NETRESOURCE *)&cEnumResourceMemory;

	sprintf(szComputerName, "\\\\%s", psCurMachine->m_szNetName);
	sprintf(szMessage, "&b&%s: \n", szComputerName);
	poApp->vPrintMessage(szMessage);

	memset(&oNetResource, 0, sizeof(oNetResource));
	lpnr->dwScope 				= RESOURCE_CONNECTED;
	lpnr->dwType 				= RESOURCETYPE_DISK;
	lpnr->dwDisplayType	= RESOURCEDISPLAYTYPE_GENERIC;
	lpnr->dwUsage 				= RESOURCEUSAGE_CONTAINER;// or maybe RESOURCEUSAGE_CONNECTABLE; 
	lpnr->lpLocalName 		= szComputerName;
	lpnr->lpRemoteName 	= szComputerName;
	lpnr->lpComment 			= NULL;
	lpnr->lpProvider 		= NULL;

	// Use Guest login and no password.  If you don't use this, you will attempt
	//   to login as the current user -- a security hole, as it will transmit
	//   your password hash over the network
	strcpy(szUserName, "Guest");
	strcpy(szPassword, "");

	memset(&Connection, 0, sizeof(NETRESOURCE));
	Connection.dwType = RESOURCETYPE_DISK;
	Connection.lpLocalName = NULL;
	Connection.lpRemoteName = szComputerName;
	Connection.lpProvider = NULL;

	Tuint ConnectRet = WNetAddConnection2(&Connection, szPassword, szUserName, 0);
	switch(ConnectRet)
	{
		case ERROR_ACCESS_DENIED: poApp->vPrintMessage("  &g&ERROR_ACCESS_DENIED\n"); break;
		case ERROR_ALREADY_ASSIGNED: poApp->vPrintMessage("  &g&ERROR_ALREADY_ASSIGNED\n"); break;
		case ERROR_BAD_DEV_TYPE: poApp->vPrintMessage("  &g&ERROR_BAD_DEVICE\n"); break;
		case ERROR_INVALID_ADDRESS: poApp->vPrintMessage("  &g&ERROR_INVALID_ADDRESS\n"); break;
		case ERROR_INVALID_PARAMETER: poApp->vPrintMessage("  &g&ERROR_INVALID_PARAMETER\n"); break;
		case ERROR_BAD_NET_NAME: poApp->vPrintMessage("  &g&ERROR_BAD_NET_NAME\n"); break;
		case ERROR_BAD_PROVIDER: poApp->vPrintMessage("  &g&ERROR_BAD_PROVIDER\n"); break;
		case ERROR_CANCELLED: poApp->vPrintMessage("  &g&ERROR_CANCELLED\n"); break;
		case ERROR_EXTENDED_ERROR: poApp->vPrintMessage("  &g&ERROR_EXTENDED_ERROR\n"); break;
		case ERROR_INVALID_PASSWORD: poApp->vPrintMessage("  &g&ERROR_INVALID_PASSWORD\n"); break;
		case ERROR_MORE_DATA: poApp->vPrintMessage("  &g&ERROR_MORE_DATA\n"); break;
		case ERROR_NO_MORE_ITEMS: poApp->vPrintMessage("  &g&ERROR_NO_MORE_ITEMS\n"); break;
		case ERROR_NO_NET_OR_BAD_PATH: poApp->vPrintMessage("  &g&ERROR_NO_NET_OR_BAD_PATH\n"); break;
		case ERROR_NO_NETWORK: poApp->vPrintMessage("  &g&ERROR_NO_NETWORK\n"); break;
		case NO_ERROR: poApp->vPrintMessage("  &g&Guest Connection Accepted\n"); break;
		default: poApp->vPrintMessage("  &g&Couldn't Login as Guest\n"); break;
	}

	if(ConnectRet != NO_ERROR)
	{
		return 0;
	}

	HANDLE hEnum = NULL;
	Tuint nWinNetReturn;
	
	nWinNetReturn = WNetOpenEnum(RESOURCE_GLOBALNET, RESOURCETYPE_ANY, 0, lpnr, &hEnum);
	if(nWinNetReturn == NO_ERROR)
	{
		psCurMachine->m_nTimeOfLastScan = nCurTime;             

		sprintf(szFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szNetName);
		pDirFile = fopen(szFilename, "wb");
		sprintf(szFilename, "%s\\temp\\index\\%s.tmn", g_aszRootPath, psCurMachine->m_szNetName);
		pNameFile = fopen(szFilename, "wb");

		while(1)
		{
			nNumResources = -1;
			nEnumResourceSize = 16384;
			memset(cEnumResourceMemory, 0, sizeof(cEnumResourceMemory));
      nWinNetReturn = WNetEnumResource(hEnum, &nNumResources, poEnumResources, &nEnumResourceSize);
			if(nWinNetReturn == NO_ERROR)
			{
				Tuint nCurResource;

				for(nCurResource = 0; nCurResource < nNumResources; nCurResource++)
				{
					sprintf(szDir, "%s\\", poEnumResources[nCurResource].lpRemoteName);
					oDirQ.vEnqueue(szDir);
					sprintf(szMessage, "  &g&(dir)&0& %s\n", szDir);
					poApp->vPrintMessage(szMessage);
				}
			}
			else if(nWinNetReturn == ERROR_NO_MORE_ITEMS)
			{
				break;
			}
			else
			{
				nWinNetReturn = GetLastError();
				if(nWinNetReturn == ERROR_MORE_DATA)
				{
					continue;
				}
				else
				{
					oDirQ.vRelease();
					fclose(pDirFile);
					fclose(pNameFile);

					sprintf(szFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szNetName);
					DeleteFile(szFilename);
					sprintf(szFilename, "%s\\temp\\index\\%s.tmn", g_aszRootPath, psCurMachine->m_szNetName);
					DeleteFile(szFilename);
					WNetCloseEnum(hEnum);
					return 0;
				}
			}
		}
	}
  WNetCloseEnum(hEnum);
	while(!oDirQ.bIsEmpty())
	{
		Tchar 						szPath[512];
		Tchar							szSearchPattern[512];
		Tchar							szFilename[512];
		WIN32_FIND_DATA		oFindData;
		HANDLE						hFind;
		Tuint							nDirIndex;
		Tuint							nFindReturn;

    oDirQ.vDequeue(szPath);
		sprintf(szSearchPattern, "%s*.*", szPath);
		hFind = FindFirstFile(szSearchPattern, &oFindData);
    if(hFind != INVALID_HANDLE_VALUE)
		{
			nDirIndex = nStoreDir(szPath, pDirFile);
			nFindReturn = 1;

			while(nFindReturn != 0)
			{
				if(oFindData.cFileName[0] != '.' && oFindData.cFileName[0] != 0)
				{
					if(oFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) 
					{  
						// Save Directory Name
						sprintf(szFilename, "%s%s\\", szPath, oFindData.cFileName);
						oDirQ.vEnqueue(szFilename);
						sprintf(szMessage, "  &g&(dir)&0& %s\n", szFilename);
						poApp->vPrintMessage(szMessage);
					}
					else
					{
						vStoreFile(oFindData.cFileName, nDirIndex, oFindData.nFileSizeLow, pNameFile);  	
						sprintf(szMessage, "               %s\n", oFindData.cFileName);
					}
				}

				nFindReturn = FindNextFile(hFind, &oFindData);
			}
		}

		FindClose(hFind);
		hFind = NULL;
	}
	if(pNameFile)
	{
		Tchar cSep = '>';

		fseek(pNameFile,-9,SEEK_END); //Put down '>' marker
		fwrite((char*)&cSep, 1, 1, pNameFile);
		fclose(pNameFile);
	}
	if(pDirFile)
	{
		fclose(pDirFile);
	}
	return 1;
}

void vIndexMachine(SsApIndexer *poApp, TMapMachine *psCurMachine)
{
	Tchar szMessage[256];
	Tuint nCurTime, nTimeSinceLastScan;

	// Check scan flags
	if(psCurMachine->m_nScanFlags == c_nScanFlagNone)
	{
		sprintf(szMessage, "&g&\\\\%s: Do Not Index\n", psCurMachine->m_szNetName);
		poApp->vPrintMessage(szMessage);
		return;
	}
	
	nCurTime = time(NULL);
	nTimeSinceLastScan = nCurTime - psCurMachine->m_nTimeOfLastScan;
	if(nTimeSinceLastScan < g_nIndexRefreshSeconds)
	{
		sprintf(szMessage, "&g&\\\\%s: Already Scanned Today\n", psCurMachine->m_szNetName);
		poApp->vPrintMessage(szMessage);
		return;
	}
	
	if(!bIndexMachine(poApp, psCurMachine))
	{
		if(nTimeSinceLastScan > g_nIndexExpireSeconds)
		{
			Tchar szFilename[260];
			sprintf(szFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szNetName);
			DeleteFile(szFilename);
			sprintf(szFilename, "%s\\temp\\index\\%s.tmn", g_aszRootPath, psCurMachine->m_szNetName);
			DeleteFile(szFilename);
			poApp->vPrintMessage("  &g&Expired machine\n");
		}
	}
}	


STRANGEAPPTHREAD(StrangeAppThread_Indexer)
{
	SsApIndexer *poApp = (SsApIndexer *)poStrangeApp;
	char szMessage[128];
	poApp->m_bIsRunning = true;
	TMapMachine *psCurMachine;
	Tuint nCurMachine = 0;
	Tuint nCurCycle = 1;

	OutputDebugString("---- Indexer Started -----\n");
	HANDLE myThread = GetCurrentThread();
	SetThreadPriority(myThread, THREAD_PRIORITY_LOWEST);

	while(1)
	{
		if(poApp->m_bHaltCommandRequested)
		{
			sprintf(szMessage, "&R&[ * &0&Indexer Service Stopped &R& * ]\n");
			poApp->vPrintMessage(szMessage);
			break;
		}

		psCurMachine = SS_CONTROL()->poGetValidMachine(nCurMachine);
		if(psCurMachine)
		{
			if(psCurMachine->m_szNetName[0] != 0)
			{
				vIndexMachine(poApp, psCurMachine);
			}
		}

		nCurMachine++;
		if(nCurMachine > SS_CONTROL()->nGetNumValidMachines())
		{
			nCurMachine = 0;
			poApp->vPrintMessage("&R&-----&0& Completed Cycle &R&-----\n\n");
			// Sleep for 30 seconds
			for(Tuint SleepCounter=0; SleepCounter<60; SleepCounter++)
			{
				if(poApp->m_bHaltCommandRequested)
				{
					break;
				}
				Sleep(500);
			}
			if(poApp->m_bHaltCommandRequested)
			{
				sprintf(szMessage, "&R&[ * &0&Indexer Service Stopped &R& * ]\n");
				poApp->vPrintMessage(szMessage);
				break;
			}
			
			poApp->vResetConsole();
			sprintf(szMessage, "&R&-----&0& Beginning Cycle %d &R&-----\n", ++nCurCycle);
			poApp->vPrintMessage(szMessage);
		}
	}

  OutputDebugString("---- Indexer Halted -----\n");

	poApp->m_bIsRunning = false;
	poApp->m_bHaltCommandRequested = false;
	_endthread();
}

void SsApIndexer::vBeginExecution()
{
	if(!m_bIsRunning)
	{
		RUN_STRANGEAPPTHREAD(StrangeAppThread_Indexer);
	}
}



//--------------------------------------------------------------------------------------------------
// $Footer: strangeapp_pinger.cpp $
//
//--------------------------------------------------------------------------------------------------


