/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsApIndexer_ftp.h"
#include "SsApIndexer_ftp_ip.h"
#include <winsock.h>

SsApIndexer *poIndexerApp = NULL;

#define MAXPACKET 4096

struct sockaddr saDestAddr;     
struct sockaddr_in saSockAddr;         // endpoint address
struct sockaddr_in saSockAddr1;        // used when bSendPort==0
struct sockaddr_in saCtrlAddr;

SOCKET ctrl_socket;
SOCKET data_socket;
SOCKET listen_socket;
int		 iCode, iMultiLine;
int		 uiFtpPort;

BOOLEAN sendingfile = FALSE;
BOOLEAN receivingfile = FALSE;
int sendingfilehandle = -1;
int receivefilehandle = -1;

char szTmpFile[4096];    // used for directory listings
char szTmp1File[4096];  // used for remote file displays

char szMsgBuf[MAXPACKET];              // main i/o buffer
char szMsgBuf2[MAXPACKET];              // main i/o buffer
u_char szSendPkt[MAXPACKET];           // output transfer buffer

Queue oFtpDirQ;
char szCurFtpDirectory[4096];
unsigned int nCurDirIndex;
FILE *pfFtpNameFile, *pfFtpDirFile;
FILE *pfLogFile;

void vPrintMessage(char *text)
{
  poIndexerApp->vPrintMessage(text);
}

void vFtpPrintf(char* text, ...)
{
	static char textBuffer[2048];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	vPrintMessage(textBuffer);
	fprintf(pfLogFile, textBuffer);
}

void vFtpPrintfln(char* text, ...)
{
	static char textBuffer[2048];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	vPrintMessage(textBuffer);
	vPrintMessage("\n");
	fprintf(pfLogFile, textBuffer);
	fprintf(pfLogFile, "\n");
}

void DoPrintf(char* text, ...)
{
	static char textBuffer[2048];
	va_list args;
	va_start(args, text);
	vsprintf(textBuffer, text, args);
	va_end(args);
	vPrintMessage(textBuffer);
	vPrintMessage("\n");
	fprintf(pfLogFile, textBuffer);
	fprintf(pfLogFile, "\n");
}

// read a reply line back in from the sockfd and return the
// value at the beginning of the first line.
int ReadLine(SOCKET sockfd)
{
  LPSTR szBuf;
  int nIndex;
  int iNumBytes,iN1,iN2; //.iN3;
  int iBytesRead;
  int iRetCode;
  int i;
 // char *s;
  char c;

  // can't do anything if we don't have a socket
  if(sockfd==INVALID_SOCKET) return(0);
  // zero our receive buffer
  memset(szMsgBuf,0,4096);
  // initialize some variables
  szBuf=szMsgBuf; iBytesRead=0; iRetCode=0;
 
 // lgk here we have a bug and it puts the system in an endless loop
 // we don't check for a socket error but one happens fix this
 iNumBytes=recv(sockfd,(LPSTR)szBuf,82,MSG_PEEK);
 if (iNumBytes == SOCKET_ERROR)
  {
   vFtpPrintf("socket recv\n");
   return (SOCKET_ERROR);
  }

  while(iBytesRead<4000 && (iNumBytes > 0))
  {
    // Trumpet WinSock Alpha 15 always returns the len (82) from a recv
    // with MSG_PEEK.  I suppose this is an error??? The spec doesn't say
    // that MSG_PEEK returns something different than normal.
    iN1=iNumBytes;
    // must terminate the string so strchr doesn't go wild.
    szBuf[iNumBytes]=0;
    // find a LF in the input if it exists
    //
    for(nIndex=0;nIndex<iNumBytes;nIndex++)
      if(szBuf[nIndex]==0 || szBuf[nIndex]==0x0a) {
        iNumBytes=nIndex+1;
        break;
      }
    // have to treat the UNISYS 5000 (EXOS driver) special.  It sends a
    // line with a CR at end and no LF and then follows it up with a
    // separate packet that has a CR/LF.  Usually this second packet is
    // not there when we peek but is when we read (answers my question
    // about the second receive containing new data!!... jaj 931024)
    if(iNumBytes>80)
      for(nIndex=0;nIndex<iNumBytes;nIndex++)
        if(szBuf[nIndex]==0x0d) {
          iNumBytes=nIndex+2;
          break;
        }
    iN2=iNumBytes;
    // otherwise read up to the full length of what the first recv saw.
    // Wonder what happens here if the second receive actually returns more
    // characters than the first receive and there was a LF in the extra data?   
    iNumBytes=recv(sockfd,(LPSTR)szBuf,iNumBytes,0);
    // again, terminate the string
    szBuf[iNumBytes]=0;
    //vFtpPrintf("[%u] readline %u - %u - %u %s",sockfd,iN1,iN2,iNumBytes,szBuf);
    // bump the receive buffer pointer
    szBuf+=iNumBytes;
    // count the bytes that we have read so far
    iBytesRead+=iNumBytes;
    // if the last character read was a LF, then stop.
    if(*(szBuf-1)==0x0a)
      break;         // '\n') break;
    // otherwise reset the timer and go read more characters
  }
  // in any case terminate what we have
  *szBuf=0;
  // find the retcode at the beginning of the line
  c=szMsgBuf[3]; szMsgBuf[3]=0; iRetCode=atoi(szMsgBuf); szMsgBuf[3]=c;
  // if the timer killed it
  while((i=strlen(szMsgBuf))>2 &&
        (szMsgBuf[i-1]==0x0a || szMsgBuf[i-1]==0x0d || szMsgBuf[i-1]==' '))
    szMsgBuf[i-1]=0;

  return(iRetCode);
}


// read a reply (may be multi line) and display in debug window
int ReadDisplayLine(SOCKET sockfd)
{
  int iRetCode;
  int iContinue;
 // char *s;
 // char c;

  // can't do anything if we don't have a socket
  if(sockfd==INVALID_SOCKET) return(0);
  // let other routine know that we are doing something right now.
  // count the lines in the response
  iMultiLine++;
  // initialize some variables
  iContinue=0;
  // go read the line
  iRetCode=ReadLine(sockfd);
  // if it wasn't a valid value or the 4th char was a hyphen
  // lgk check for error here and return if so
  if (iRetCode == SOCKET_ERROR)
    {
	 return  iRetCode;
    }
  // lgk bug here when iretcode == 0 does not mean continue

  if(((iRetCode<100) && (iRetCode > 0)) || iRetCode>599 || szMsgBuf[3]=='-')
    // then it is a continuation line
    iContinue=1;
  // lgk do not add blank lines to the window
  //if (strlen(szMsgBuf) != 0)  
  // send the line we read to our user/debug window
			//vFtpPrintf("%s\n", szMsgBuf);

  //  vFtpPrintf("iRetCode=%u,  =%u",iRetCode,iContinue);
  // if the timer killed it
  // we only want to set the real return code in certain situations
  if((iMultiLine==1 || iCode==0) && iRetCode>99 && iRetCode<600)
    iCode=iRetCode;
  // handle continuation lines
  // dont do this if we have a null line

  if(iContinue==1 || (iCode>0 && iMultiLine>1 && iRetCode!=iCode))
    ReadDisplayLine(sockfd);
  // count back down our multiline reponses
  iMultiLine--;
  // return only the first char of return code
 // lgk special case here to return 550 not first char
 if (iCode==550)
   return iCode;
 if(iCode>99 && iCode<600)
    return (iCode/100);
  else return 0;
}


// user close routine
volatile SOCKET DoClose(SOCKET sockfd)
{
LINGER ntlinger;
 
  if(sockfd!=INVALID_SOCKET) {
  if(WSAIsBlocking())
    {      
      vFtpPrintf("(%u) Cancelled blocking call\n",sockfd);
      WSACancelBlockingCall();
    }
 // dont report error here since shutdown does not work on non data sockets
//    if(shutdown(sockfd,2)==SOCKET_ERROR)
//      ReportWSError("shutdown",WSAGetLastError());
   
   shutdown(sockfd,2);

//  93.12.04 - so Lanera Winsock Works
   ntlinger.l_onoff  = (u_short)TRUE;
    ntlinger.l_linger = (u_short)0;
    setsockopt(sockfd,SOL_SOCKET,SO_LINGER,
                (LPSTR)&ntlinger,sizeof(ntlinger) );

    if(closesocket(sockfd)==SOCKET_ERROR)
      vFtpPrintf("closesocket error\n");
    else {
            //vFtpPrintf("[%u] Socket closed.\n",sockfd);
            sockfd=INVALID_SOCKET;
    }
  }

  if(sockfd!=INVALID_SOCKET)
    vFtpPrintf("%u Failed to close socket.",sockfd);

  return(sockfd);
}

int SendPacket(SOCKET sockfd,LPSTR msg)
{
  int i;

  if(sockfd==INVALID_SOCKET) return(-1);
  i=strlen(msg);
  strcpy((char *)szSendPkt,msg);
  // append a CRLF to the end of outgoing messages
  szSendPkt[i++]='\r';
  szSendPkt[i++]='\n';
  szSendPkt[i]=0;  
  i=sendstr(sockfd,(char *)szSendPkt,i);
  return (i);
}

int getreply(SOCKET ctrl_skt,LPSTR cmdstring)
{
  int iRetCode=0;

  iCode=0;
  //vFtpPrintf(cmdstring);	
	//vFtpPrintf("\n");

	if(SendPacket(ctrl_skt,cmdstring)!=-1)
	{
    iRetCode=ReadDisplayLine(ctrl_skt);
  }
  return(iRetCode);  // 0 - 5
}

int command(SOCKET ctrl_skt, char *fmt,...)
{
   va_list args;
   char szBuf[256];
 //  int  iRetCode;

   va_start(args,fmt);
   vsprintf(szBuf,fmt,args);
   va_end(args);
   return(getreply(ctrl_skt,szBuf));
}

int qcommand(SOCKET ctrl_skt, char *fmt,...)
{
   va_list args;
   char szBuf[256];
 //  int  iRetCode;

   va_start(args,fmt);
   vsprintf(szBuf,fmt,args);
   va_end(args);
   command(ctrl_skt,szBuf);
   return iCode ;
}

// process CWD
int DoCWD(SOCKET ctrl_skt,LPSTR path)
{
	if(!path || !ctrl_skt)
	{
		return 5;
	}

  if(command(ctrl_skt,"CWD %s",path)==FTP_ERROR && iCode==500) {
    command(ctrl_skt,"XCWD %s",path);
  }
  return(iCode/100);
}

// proces PWD
int DoPWD(SOCKET ctrl_skt)
{
  if(command(ctrl_skt,"PWD")==FTP_ERROR && iCode==500) {
    command(ctrl_skt,"XPWD");
  }
  return(iCode/100);
}

volatile SOCKET GetFTPListenSocket(SOCKET ctrl_skt)
{
    SOCKET listen_skt;
    int iLength;
    int iRetCode;
    char *a,*p;
    int iFlag=1;

    // create a data socket
    if((listen_skt=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP))==INVALID_SOCKET)
    {
        vFtpPrintf("socket create error\n");
        return (INVALID_SOCKET);
    }
    // let system pick an unused port. we tell remote end with PORT cmd
 
    /*DoPrintf("[%u] going to listen %s port %u",listen_skt,
      inet_ntoa(saCtrlAddr.sin_addr),ntohs(saCtrlAddr.sin_port));*/

		int bSendPort = 1;
    if(bSendPort) {
      saCtrlAddr.sin_port=htons(0);
      saCtrlAddr.sin_family=AF_INET;
      saCtrlAddr.sin_addr.s_addr=0;
    } else
      // otherwise we attempt to reuse our ctrl_skt
      if(setsockopt(listen_skt,SOL_SOCKET,SO_REUSEADDR,
         (char *)&iFlag,sizeof(iFlag))==SOCKET_ERROR)
      {
        vFtpPrintf("setsockopt error\n");
        closesocket(listen_skt);
        return(INVALID_SOCKET);
      }
    //  Bind name to socket
    if( bind((SOCKET)listen_skt,(LPSOCKADDR)&saCtrlAddr,
             (int)sizeof(struct sockaddr))==SOCKET_ERROR)
      {
        vFtpPrintf("bind error\n");
        closesocket(listen_skt);
        return (INVALID_SOCKET);
    }
    // get the port name that we got for later transmission in PORT cmd
    iLength=sizeof(saCtrlAddr);
    if(getsockname(listen_skt,(struct sockaddr *)&saCtrlAddr,&iLength)<0)
    {
      vFtpPrintf("getsockname error\n");
      closesocket(listen_skt);
      return(INVALID_SOCKET);
    }
    // invoke listener
    if(listen(listen_skt,1)!=0)
    {
      vFtpPrintf("listen error\n");
       closesocket(listen_skt);
      return(INVALID_SOCKET);
    }

// inform remote end about our port that we created.
    if(bSendPort)
    {
      struct sockaddr_in saTmpAddr;
      int iLength;

      iLength = sizeof (saTmpAddr);
      if (getsockname(ctrl_skt,(LPSOCKADDR)&saTmpAddr, &iLength)
        ==SOCKET_ERROR)
      {
        vFtpPrintf("getsockname error\n");
      }

      a = (char *)&saTmpAddr.sin_addr;
      p = (char *)&saCtrlAddr.sin_port;
#define  UC(b)  (((int)b)&0xff)
      if((iRetCode=command(ctrl_skt,"PORT %d,%d,%d,%d,%d,%d",
          UC(a[0]), UC(a[1]), UC(a[2]), UC(a[3]),
          UC(p[0]), UC(p[1])))!=FTP_COMPLETE) {
        //DoPrintf("[%u] remote end didn't understand our port command.",listen_skt);
        return(listen_skt);
      }
    }
    /*DoPrintf("[%u] listener %s port %u",listen_skt,
      inet_ntoa(saCtrlAddr.sin_addr),ntohs(saCtrlAddr.sin_port));*/
    return(listen_skt);
}

// read information from the data socket into a file.  
int ReadMass(SOCKET sockfd,LPSTR filename, LPSTR shortname, BOOL binaryflag)
{
  int  iNumBytes;
  int  iRetCode;
  int  iFileHandle;
  long lBytesRead;
  time_t ttStart,ttStop;

  // if we don't have a socket, return an error  
  if(sockfd==INVALID_SOCKET) return 0;
  // initialize some vars
  lBytesRead=0l; iRetCode=0; 
  // at the moment we are ignoring the fact that the local destination file
  // may not open correctly.
  if((iFileHandle=_lcreat(filename,0))== -1)
    {
     DoPrintf("failed to open file %s (%u)",filename,errno);
	 DoPrintf("Trying shorting name %s ",shortname);
	 if ((iFileHandle=_lcreat(shortname,0)) == -1)
	    DoPrintf("Still failed to open file %s (%u)",shortname,errno);
	 }
	// here if the name was too long than may be an ntfs partition so try the shorter name

    receivefilehandle = iFileHandle;
    receivingfile = TRUE;

  // get the start time
  ttStart=time(NULL);
  // loop to receive input from remote end
  while((iNumBytes=recv(sockfd,(LPSTR)szMsgBuf,4000,0)) >0)
  {
    // write what we received if the file is open
    if(iFileHandle!= -1)
      _lwrite(iFileHandle,szMsgBuf,iNumBytes);
    // count the characters that we received
    // update screen
  }
  // get the finish time
  ttStop=time(NULL);
  // if the output file is open, close it
  if(iFileHandle != -1)
   {
     _lclose(iFileHandle);
    receivingfile = FALSE;
    receivefilehandle = -1;
   }
   
  // if we had a recv error, let us know about it
  if(iNumBytes==SOCKET_ERROR)
  {
    vFtpPrintf("recv error\n");
    if(lBytesRead==0l)
    {
      return(FALSE);
    }
  }

  /*ttDiff=ttStop-ttStart;
  if(ttDiff==0l) ttDiff=1l;
  // show the user how we did
   if (lBytesRead > 1024)
   {
       DoPrintf("%ld bytes received in %.2f seconds, (%3.2fKbytes/sec), transfer %s",
       lBytesRead,(float)ttDiff,(float)lBytesRead/1024.000/(float)ttDiff,
     (iFileHandle==-1)?"failed":"succeeded");

   	 // lgk set global variable so that we know it succeeded so we will 
	 // close window
	 }

   else 
	   {
  DoPrintf("Received %ld bytes in %ld secs, %ld bps, transfer %s",
    lBytesRead,(long)ttDiff,
    lBytesRead*8l/(long)ttDiff,
    (iFileHandle==-1)?"failed":"succeeded");
	 }*/

 	 // lgk set global variable so that we know it succeeded so we will 
	 // close window

   // DoPrintf("Bell = %d \n",bBell);
 
  return (TRUE);
}

int RetrieveFile(SOCKET ctrl_skt,LPSTR szCMD,LPSTR localfile, LPSTR shortname,char rtype)
{
  int iRetCode;
  int iLength;
  int iFlag = 1;
	char cType;

  iCode=0;
  iMultiLine = 0;
  // if we don't have a valid control socket, can't do anything
  if(ctrl_skt==INVALID_SOCKET) 
	{
    vFtpPrintfln("no ctrl_skt, ignored");
    return(0);
  }
  // if we are doing something, don't try to do this
  // if the requested type is not the same as the default type
  command(ctrl_skt,"TYPE %c",rtype);
  cType=rtype;

  // create a listener socket, if it is successful
  if((listen_socket=GetFTPListenSocket(ctrl_skt))!=INVALID_SOCKET)
     {
    // send command to see the result of this all
    iRetCode=command((SOCKET)ctrl_skt,szCMD);
    // read the control channel (should return 1xx if it worked)
    if(iRetCode==FTP_PRELIM) {
      // get our data connection
      iLength=sizeof(saSockAddr1);
      data_socket=accept(listen_socket,(struct sockaddr far *)&saSockAddr1,
                         (int far *)&iLength);
    // lgk check for error case here
	if (data_socket == INVALID_SOCKET)
	  {
							vFtpPrintf("accept for receiving error\n");
             data_socket=DoClose(data_socket);
             return 0;
	  }
       
      if(setsockopt(data_socket,SOL_SOCKET,SO_REUSEADDR,
         (char *)&iFlag,sizeof(iFlag))==SOCKET_ERROR)
      {
        vFtpPrintf("setsockopt error\n");
        closesocket(data_socket);
        return(INVALID_SOCKET);
      }
  
      // if it failed, we have to quit this
      if(data_socket==INVALID_SOCKET) {
        vFtpPrintf("accept error\n");
				DoClose(listen_socket);
        iRetCode=0;
      } else {
        // we don't need the listener socket anymore
        listen_socket=DoClose(listen_socket);
        // inform user of the connection
        //DoPrintf("[%u] accept from %s port %u", data_socket,
          //inet_ntoa(saSockAddr1.sin_addr),ntohs(saSockAddr1.sin_port));
        // copy the file
        iRetCode=ReadMass(data_socket,localfile,shortname,0);
        // shut the data socket down
        if(shutdown(data_socket,2)!=0)
          vFtpPrintf("shutdown error\n");
        // close the data socket
        data_socket=DoClose(data_socket);
        // read the close control message (should return 2xx)
        iRetCode=ReadDisplayLine(ctrl_skt);
      }
    } else {
      listen_socket=DoClose(listen_socket);
      iRetCode=0;
    }

  } else {
    listen_socket=DoClose(listen_socket);
    iRetCode=0;
  }
  return(iRetCode);
}

unsigned char bIsDirectory(char *szDirectoryLine)
{
	return(szDirectoryLine[0] == 'd');
}

char *vGetFilename(char *szLine)
{
	char *p;

	p = szLine;
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
  return(p);
}

unsigned int nGetSize(char *szDirectoryLine)
{
	char *p;

	p = szDirectoryLine;
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	while(*p != ' ' && *p != 0)	{ p++; }	// Get to first whitespace
	while(*p == ' ' && *p != 0)	{ p++; }	// Skip first whitespace
	return atoi(p);
}

int DoDirList(SOCKET ctrl_skt,LPSTR szCMD)
{
  int nRC;
	char buffer[256];
  nRC=RetrieveFile(ctrl_skt,szCMD,szTmpFile,szTmpFile,'A');

	FILE *f = fopen(szTmpFile, "r");
	if(f!=NULL)
	{
		char *p, *pszFilename;
		unsigned int nSize;
		fgets(buffer, 256, f);
		while(!feof(f))
		{
			if(buffer[0] == 'd' || buffer[0] == 'f' || buffer[0] == '-')
			{
				// Throw out \n or \r
				p = buffer;
				while(*p != 0)
				{
					if(*p == 13 || *p == 10)
					{
						*p = 0;
						break;
					}
					p++;
				}

				if(bIsDirectory(buffer))
				{
					pszFilename = vGetFilename(buffer);
					if(pszFilename[0] != '.')
					{
						//vFtpPrintf("      (dir) %s\n", pszFilename);
						char szNewDir[512];
						sprintf(szNewDir, "%s%s/", szCurFtpDirectory, pszFilename);
						oFtpDirQ.vEnqueue(szNewDir);
					}
				}
				else
				{
					pszFilename = vGetFilename(buffer);
					nSize = nGetSize(buffer);
					//vFtpPrintf("      %s (%d bytes)\n", pszFilename, nSize);
					vStoreFile(pszFilename, nCurDirIndex, nSize, pfFtpNameFile);
				}
			}
			fgets(buffer, 256, f);
		}
		fclose(f);
	}
  return(nRC);
}

SOCKET nDoConnect(char *szUsername, char *szPassword, char *szHostname, int nPort)
{
	int iLength,iRetCode,iMultiLine,iCode;
  int iFlag=1;
	SOCKET ctrl_skt;
	SOCKADDR_IN saCtrlAddr;
	
	if((ctrl_skt = connectTCP(szHostname, "ftp")) == INVALID_SOCKET)
	{
		vFtpPrintf("Connection to %s failed\n", szHostname);
		return INVALID_SOCKET;
	}


  // get information about local end of the connection
  iLength = sizeof (saCtrlAddr);
  if (getsockname(ctrl_skt,(struct sockaddr *)&saCtrlAddr, &iLength)
      ==SOCKET_ERROR)
  {
		vFtpPrintf("Get Socket Name failed\n");
		if(WSAIsBlocking())
		{
			WSACancelBlockingCall();
		}
    shutdown(ctrl_skt, 2);
    return(INVALID_SOCKET);
  }
  // show remote end address
  //vFtpPrintf("[%u] from %s port %u",ctrl_skt, inet_ntoa(saCtrlAddr.sin_addr), ntohs(saCtrlAddr.sin_port));

  ctrl_socket = ctrl_skt;
  iMultiLine = 0;
  iCode = 0;

	while((iRetCode=ReadDisplayLine(ctrl_skt)) == FTP_PRELIM);

  if(iRetCode==FTP_COMPLETE) {
    if (setsockopt(ctrl_skt, SOL_SOCKET, SO_OOBINLINE, (LPSTR)&iFlag, sizeof(iFlag))==SOCKET_ERROR)
    {
			vFtpPrintf("setsockopt failed\n");
    }

    // send our userid to the ftp_host
    if((iRetCode=command(ctrl_skt,"USER %s",szUsername))==FTP_CONTINUE) // || 1)
    {
      // if the remote system requires a password, send it.
      iRetCode=command(ctrl_skt,"PASS %s",szPassword);
    }
    // if we are successfully logged on,.....
    if(iRetCode!=FTP_COMPLETE) // || 0)
    {
			vFtpPrintf("Logon with USER %s PASS %s failed\n", szUsername, szPassword);
			shutdown(ctrl_skt, 2);
      vFtpPrintfln("logon failure, so quitting");
			return(INVALID_SOCKET);
    }
  } 
	else 
	{
    vFtpPrintf("unk open msg \"%s\" %u",szMsgBuf,iCode);
    // allow other processes to work
		shutdown(ctrl_skt, 2);
    return(INVALID_SOCKET);
  }
  return (ctrl_skt);
}

void vIndexSite(char *szUsername, char *szPassword, char *szHostname, int nPort)
{
	char szDirectory[512];
	unsigned int nFtpRetCode;

	while(!oFtpDirQ.bIsEmpty())
	{
		oFtpDirQ.vDequeue(szDirectory);
		strcpy(szCurFtpDirectory, szDirectory);
		nCurDirIndex = nStoreDir(szDirectory, pfFtpDirFile);

		vFtpPrintf("   %s\n", szDirectory);
		if((nFtpRetCode = DoCWD(ctrl_socket, szDirectory)) == FTP_COMPLETE)
		{
			DoDirList(ctrl_socket, "LIST");
		}
		else
		{
			if(nFtpRetCode == FTP_RETRY)
			{
				vFtpPrintf("* Attempting Reconnet...\n");
				if((ctrl_socket = nDoConnect(szUsername, szPassword, szHostname, nPort)) != INVALID_SOCKET)
				{
					oFtpDirQ.vEnqueue(szCurFtpDirectory);
				}
			}
		}
	}
}

Tboolean bIndexFtpSite(SsApIndexer *poApp, char *szFilename, char *szUsername, char *szPassword, char *szHostname, int nPort)
{
	uiFtpPort = nPort;
	WSADATA WSAData;                       
	WSAStartup((WORD)0x0101, &WSAData);
	char szString[256], *pStr;
	unsigned int nRC;
	char szIndexFilename[256];

  poIndexerApp = (SsApIndexer *)poApp;

	sprintf(szIndexFilename, "%s\\temp\\index\\%s.log", g_aszRootPath, szFilename);
	pfLogFile = fopen(szIndexFilename, "w");
	
	GetTempFileName(".","DIR",0,szTmpFile);
  FILE *pfTmpFile = fopen(szTmpFile, "wb");
  fclose(pfTmpFile);

	vFtpPrintf("-------------------------------------------------------\n");
	if((ctrl_socket = nDoConnect(szUsername, szPassword, szHostname, nPort)) != INVALID_SOCKET)
	{
		sprintf(szIndexFilename, "%s\\temp\\index\\%s.tmn", g_aszRootPath, szFilename);
		pfFtpNameFile = fopen(szIndexFilename, "wb");
		sprintf(szIndexFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, szFilename);
		pfFtpDirFile = fopen(szIndexFilename, "wb");


		nRC = DoPWD(ctrl_socket);
    if(nRC==FTP_COMPLETE) {
      if((pStr=strchr(szMsgBuf,'"'))!=NULL)
        strncpy(szString,++pStr,180);
      if((pStr=strchr(szString,'"'))!=NULL)
        *pStr=0;
		}
		oFtpDirQ.vEnqueue("/");
		vIndexSite(szUsername, szPassword, szHostname, nPort);
		command(ctrl_socket, "QUIT");
	}
  else
  {
    fclose(pfLogFile);
    _unlink(szTmpFile);
    WSACleanup();
    return 0;
  }
	if(pfFtpNameFile)
	{
		Tchar cSep = '>';

		fseek(pfFtpNameFile,-9,SEEK_END); //Put down '>' marker
		fwrite((char*)&cSep, 1, 1, pfFtpNameFile);
		fclose(pfFtpNameFile);
	}
	if(pfFtpDirFile)
	{
		fclose(pfFtpDirFile);
	}
	vFtpPrintf("-------------------------------------------------------\n");
  _unlink(szTmpFile);

	fclose(pfLogFile);

	WSACleanup();

  return 1;
}