/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsBaCommon.h"
#include "SsApIndexer_ftp_ip.h"
#include "SsApIndexer_ftp.h"

SOCKET connectTCP(char *host,char *service)
{
  return connectsock(host,service,"tcp");
}

SOCKET connectUDP(char *host,char *service)
{
  return connectsock(host,service,"udp");
}

SOCKET connectsock(char *host,char *service,char *protocol)
{
  struct hostent *pHostEntry;      // pointer to host entry
  struct servent FAR *pServiceEntry;   // pointer to service entry
  struct protoent FAR *pProtoEntry;    // pointer to protocol entry
  volatile SOCKET sSocket;                  // socket
  int nSocketType;                 // socket type
  short pproto;
  int iFlag=1;

  vFtpPrintf("  ");
  // initialize socket address structure
  memset((char *)&saSockAddr,0,sizeof(saSockAddr));
  saSockAddr.sin_family=AF_INET;
  // get service port number from name
  if(pServiceEntry=getservbyname(service,protocol))
    {
    if (uiFtpPort != 21)
      {
       vFtpPrintfln("Overridding Ftp port number with %u",uiFtpPort);
       saSockAddr.sin_port=htons((u_short)uiFtpPort);
      }
    else saSockAddr.sin_port=pServiceEntry->s_port;
    }
 else if((saSockAddr.sin_port=htons((u_short)atoi(service)))==0)
    {
//  lgk new code first try user interface setting if different than 21
    if (uiFtpPort != 21)
      {
       vFtpPrintfln("Overridding Ftp port number with %u",uiFtpPort);
       saSockAddr.sin_port=htons((u_short)uiFtpPort);
      }
    else
     {
   
       if(strcmpi(service,"FTP")==0)
          saSockAddr.sin_port=htons(21);
       else {
          vFtpPrintfln("can't get \"%s\" service entry",service);
         return INVALID_SOCKET;
       }
     }
  }
  // map host to ip, allow ip
  if(pHostEntry=gethostbyname(host))
    memcpy((char *)&saSockAddr.sin_addr,pHostEntry->h_addr,
             pHostEntry->h_length);
//  memcpy(pHostEntry->h_addr,(char *)&saSockAddr.sin_addr,pHostEntry->h_length);
  else if((saSockAddr.sin_addr.s_addr=inet_addr(host))==INADDR_NONE) {
    vFtpPrintfln("can't get \"%s\" host entry",host);
    return INVALID_SOCKET;
  }
  // map protocol name to protocol number
  if((pProtoEntry=getprotobyname(protocol))==0) {
    if(strcmpi(protocol,"TCP")==0)
      pproto=6;
    else {
      vFtpPrintfln("can't get \"%s\" protocol entry",protocol);
      return INVALID_SOCKET;
    }
  } else pproto=pProtoEntry->p_proto;
  // use protocol to choose socket type
  if(strcmp(protocol,"udp")==0)
    nSocketType=SOCK_DGRAM;
  else
    nSocketType=SOCK_STREAM;
  // allocate a socket
  sSocket=socket(AF_INET,nSocketType,pproto);
  if(sSocket==INVALID_SOCKET) 
  {
    vFtpPrintfln("create socket error\n");
    return INVALID_SOCKET;
  }
      // lgk set for reuse
      if(setsockopt(sSocket,SOL_SOCKET,SO_REUSEADDR,
         (char *)&iFlag,sizeof(iFlag))==SOCKET_ERROR)
      {
        vFtpPrintf("setsockopt error\n");
        closesocket(data_socket);
        return INVALID_SOCKET;
      }
   
  memcpy((LPSTR)&saSockAddr1,(LPSTR)&saSockAddr,sizeof(saSockAddr));
  saSockAddr1.sin_port=htons(20);

  vFtpPrintfln("connecting to %s ...",inet_ntoa(saSockAddr.sin_addr));
  // connect the socket
  if(connect(sSocket,(struct sockaddr *)&saSockAddr,sizeof(saSockAddr))
      ==SOCKET_ERROR) {
    vFtpPrintfln("Can't connect to %s %s service port %u.",host,service,ntohs(saSockAddr.sin_port));
    return INVALID_SOCKET;
  }

  vFtpPrintfln("Connected to %s port %u",
    inet_ntoa(saSockAddr.sin_addr),ntohs(saSockAddr.sin_port));

  return sSocket;
}

int sendstr(SOCKET sSocket,LPSTR pBuf,int nBytesToWrite)
{
  int nBytesLeft,nWritten;

  nBytesLeft=nBytesToWrite;
  while(nBytesLeft > 0) {                   // while we haven't written enough
    nWritten=send(sSocket,pBuf,((nBytesLeft>1024)?1024:nBytesLeft),0); // write what we can
    // vFtpPrintf("[%u] %u send %u",sSocket,nBytesLeft,nWritten);
  if(nWritten <= 0)
    {
      //vFtpPrintfln("[%u] %u send %u",sSocket,nBytesLeft,nWritten);
      return(nWritten);  // error occured
    }
    nBytesLeft -= nWritten;                 // count what we wrote
    pBuf   += nWritten;                     // adjust buffer pointer
  }
  return(nBytesToWrite - nBytesLeft);       // return count of bytes written
}

