/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsBaCommon.h"
#include "apps/SsApMaint.h"
#include "SsStControl.h"

struct TValidException
{
	Tchar m_szIpAddress[16];
	Tchar m_szName[17];
	Tuint	m_nScanFlags;
};

struct TMachineException
{
	Tchar m_szIpAddress[16];
	Tchar m_szBaseFtpURL[128];
};

TValidException *psValidExceptions;
TMachineException *psMachineExceptions;
Tuint nValidExceptions, nMachineExceptions;

// -------------------------------------------------------------------------------------------										
// Utility Functions

Tuint nFindEndOfString(char *pszText) //finds index of next '|' in a string
{
	Tint nCurChar = 0;

	while(nCurChar < 5000) // For safety
	{
		if(pszText[nCurChar] == '<' || pszText[nCurChar] == '>')	return(nCurChar);
		nCurChar++;
	}
	return 0;
}

Tboolean IsNum(char check)	//is the ascii character a number
{
	if(check>47 && check< 58)
		return(True);
	return(False);
}

Tuint nExtensionToFile(char *Ext, Tuint StringLen)
{
	//music,pic,movie,warez
	char TheExt[256];
	Tuint i=0,j,p,q;		

	if(StringLen<12)	//small file name
		return(NUMDATABASES-1);

	j=StringLen-9;

	while(j>0)	//get position of LAST '.' in string
	{
		if(Ext[j]=='.')
			break;
		j--;
	}
	if(j==0)
		return(NUMDATABASES-1);
	ASSERT(((StringLen-9)-j)<256);
	memset(TheExt,0,256);memcpy(TheExt,&Ext[j],(StringLen-9)-j); //get just the extension into a string
	strlwr(TheExt);
	for(Tuint x=0;x<NUMDATABASES;x++)
	{
		for(Tuint y=0;y<g_anNumEntries[x];y++)
		{
			p=strlen(g_aszExtensions[i]);
			q=strlen(TheExt);
			if(p==q)
			{
				if(strncmp(g_aszExtensions[i],TheExt,p)==0)
					return(x);
				if(strncmp(g_aszExtensions[i],".*##",4)==0)//check for numeric extensions (like part of an archive)
				{
					if(IsNum(TheExt[2]) && IsNum(TheExt[3]))
						return(3);
				}

			}
			i++;
		}
	}
	return(NUMDATABASES-1);
}

// Read an entire name and trailing information, return length read (or 0 if EOF)
Tuint nReadName(Tchar *pszBuffer, FILE *pFile)
{
	Tuint nCurChar = 0;

	if(feof(pFile))	return(0);

	while(1)
	{
		pszBuffer[nCurChar] = fgetc(pFile);
		if(feof(pFile))	return(0);
		if(pszBuffer[nCurChar] == '<' || pszBuffer[nCurChar] == '>')
		{
			nCurChar++;
			fread((void *)&pszBuffer[nCurChar], 8, 1, pFile);
			return(nCurChar+8);
		}
		nCurChar++;
	}
}
// -------------------------------------------------------------------------------------------										

// Tboolean bShutdownAllScan()				Force all scanning applications to close
Tboolean bShutdownAllScan()
{
  gs_poSsControl->poGetStrangeControl()->bSearchingAllowed = 0;

	Tboolean bAppRunning;
	for(Tuint nStopCount = 0; nStopCount < 100; nStopCount++)
	{
		bAppRunning = False;
		if(SS_CONTROL()->bAppRunning(c_nSsStControlAppPinger))				{ SS_CONTROL()->vStopApp(c_nSsStControlAppPinger); bAppRunning = True; }
		if(SS_CONTROL()->bAppRunning(c_nSsStControlAppPingerMaint))		{ SS_CONTROL()->vStopApp(c_nSsStControlAppPingerMaint); bAppRunning = True; }
		if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIndexer))				{ SS_CONTROL()->vStopApp(c_nSsStControlAppIndexer); bAppRunning = True; }
		if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIpMapperMaint))	{ SS_CONTROL()->vStopApp(c_nSsStControlAppIpMapperMaint); bAppRunning = True; }	
    if(SS_CONTROL()->bAppRunning(c_nSsStControlAppSearchMonitor)) { SS_CONTROL()->vStopApp(c_nSsStControlAppSearchMonitor); bAppRunning = True; }
		if(!bAppRunning)
		{
			break;
		}
		Sleep(1000);
	}

	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppPinger))				{ return False; }
	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppPingerMaint))		{ return False; }
	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIndexer))				{ return False; }
	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIpMapperMaint))	{ return False; }
  if(SS_CONTROL()->bAppRunning(c_nSsStControlAppSearchMonitor))	{ return False; }


	return True;
}

/////////////////////////////////////////////////////////////////////////////////
#if 0
// void vGenerateValidMachinesFromHugeMachines()				Generate a list of valid machines from the huge machine file
void vGenerateValidMachinesFromHugeMachines(SsApMaint *poApp)
{
	Tchar szFilename[128];
	Tchar szMessage[128];
	FILE *fValidMachines, *fValidExceptions;
  TMapMachine *psCurMachine;
	TMapMachine sValidMachine;
	Tuint nCurMachine = 0;
	Tuint nValidMachineCount = 0;
	TValidException *psValidExceptions = NULL;
	Tuint nValidExceptions = 0;
	Tchar szExceptionBuffer[128];

	sprintf(szFilename, "%s\\data\\ValidException.txt", g_aszRootPath);
	fValidExceptions = fopen(szFilename, "r");
	if(fValidExceptions)
	{
		fgets(szExceptionBuffer, 128, fValidExceptions);
		nValidExceptions = atoi(szExceptionBuffer);
		psValidExceptions = new TValidException [nValidExceptions];

		for(Tuint nCurException=0; nCurException<nValidExceptions; nCurException++)
		{
			fgets(szExceptionBuffer, 128, fValidExceptions);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psValidExceptions[nCurException].m_szIpAddress, szExceptionBuffer);
			fgets(szExceptionBuffer, 128, fValidExceptions);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psValidExceptions[nCurException].m_szName, szExceptionBuffer);
			fgets(szExceptionBuffer, 128, fValidExceptions);
			psValidExceptions[nCurException].m_nScanFlags = atoi(szExceptionBuffer);
		}	
		fclose(fValidExceptions);
	}

	sprintf(szFilename, "%s\\data\\ValidMachineMap.dat", g_aszRootPath);
	fValidMachines = fopen(szFilename, "wb");
	if(fValidMachines == NULL)
	{
		OutputDebugString("failed\n");
		ASSERT(0);
	}

	for(nCurMachine=0; nCurMachine<SS_CONTROL()->nGetNumMachines(); nCurMachine++)
	{
		psCurMachine = SS_CONTROL()->poGetHugeMachine(nCurMachine);
		if(strstr(psCurMachine->m_szDomain, g_aszValidDomainSubstr) != NULL)
		{
			sValidMachine.m_bMachineUp = psCurMachine->m_bMachineUp;
			sValidMachine.m_nTimeOfLastNamePing = psCurMachine->m_nTimeOfLastNamePing;
			sValidMachine.m_nTimeOfLastScan = psCurMachine->m_nTimeOfLastScan;
			sValidMachine.m_nHugeMachineIndex = psCurMachine->m_nHugeMachineIndex;
			strcpy(sValidMachine.m_szDomain, psCurMachine->m_szDomain);
			strcpy(sValidMachine.m_szIpAddress, psCurMachine->m_szIpAddress);
			strcpy(sValidMachine.m_szNetName, psCurMachine->m_szNetName);
			sValidMachine.m_nScanFlags = 1;
			sprintf(szMessage, "Valid Machine: %s\n", psCurMachine->m_szNetName);
			poApp->vPrintMessage(szMessage);

			for(Tuint nCurException=0; nCurException<nValidExceptions; nCurException++)
			{
				if(strcmp(psValidExceptions[nCurException].m_szIpAddress, psCurMachine->m_szIpAddress) == 0)
				{
					strcpy(sValidMachine.m_szNetName, psValidExceptions[nCurException].m_szName);
					sValidMachine.m_nScanFlags = psValidExceptions[nCurException].m_nScanFlags;

					strcpy(psCurMachine->m_szNetName, psValidExceptions[nCurException].m_szName);
					psCurMachine->m_nScanFlags = psValidExceptions[nCurException].m_nScanFlags;

					sprintf(szMessage, "  &b&*&0& Proccessed Exception %s -> %s\n", sValidMachine.m_szIpAddress, sValidMachine.m_szNetName);
					poApp->vPrintMessage(szMessage);
					break;
				}
			}	

			fwrite((void *)&sValidMachine, sizeof(TMapMachine), 1, fValidMachines);
			nValidMachineCount++;
		}
	}
	fclose(fValidMachines);

	if(psValidExceptions)
	{
		delete [] psValidExceptions;
	}
}
#endif
/////////////////////////////////////////////////////////////////////////////////

void vProcessMachineExceptions(SsApMaint *poApp)
{
	Tchar szFilename[128];
	Tchar szMessage[128];
	FILE *fValidExceptions;
  TMapMachine *psCurMachine;
	TMapMachine sValidMachine;
	Tuint nCurMachine = 0;
	TValidException *psValidExceptions = NULL;
	Tuint nValidExceptions = 0;
	Tchar szExceptionBuffer[128];
	
	// Read in all valid exceptions (if any)
	sprintf(szFilename, "%s\\data\\ValidException.txt", g_aszRootPath);
	fValidExceptions = fopen(szFilename, "r");
	if(fValidExceptions) 
	{
		fgets(szExceptionBuffer, 128, fValidExceptions);
		nValidExceptions = atoi(szExceptionBuffer);
		psValidExceptions = new TValidException [nValidExceptions];
		
		for(Tuint nCurException=0; nCurException<nValidExceptions; nCurException++)
		{
			fgets(szExceptionBuffer, 128, fValidExceptions);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psValidExceptions[nCurException].m_szIpAddress, szExceptionBuffer);
			fgets(szExceptionBuffer, 128, fValidExceptions);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psValidExceptions[nCurException].m_szName, szExceptionBuffer);
			fgets(szExceptionBuffer, 128, fValidExceptions);
			psValidExceptions[nCurException].m_nScanFlags = atoi(szExceptionBuffer);
		}	
		fclose(fValidExceptions);
	}
		
	for(nCurMachine=0; nCurMachine<SS_CONTROL()->nGetNumMachines(); nCurMachine++)
	{
		psCurMachine = SS_CONTROL()->poGetValidMachine(nCurMachine);
			
		for(Tuint nCurException=0; nCurException<nValidExceptions; nCurException++)
		{
			if(strcmp(psValidExceptions[nCurException].m_szIpAddress, psCurMachine->m_szIpAddress) == 0)
			{
				strcpy(sValidMachine.m_szNetName, psValidExceptions[nCurException].m_szName);
				sValidMachine.m_nScanFlags = psValidExceptions[nCurException].m_nScanFlags;
				
				strcpy(psCurMachine->m_szNetName, psValidExceptions[nCurException].m_szName);
				psCurMachine->m_nScanFlags = psValidExceptions[nCurException].m_nScanFlags;
				
				sprintf(szMessage, "  &b&*&0& Proccessed Exception %s -> %s\n", sValidMachine.m_szIpAddress, sValidMachine.m_szNetName);
				poApp->vPrintMessage(szMessage);
				break;
			}
		}	
	}
	
	if(psValidExceptions)
	{
		delete [] psValidExceptions;
	}
}

// void vPurgeInvalidDataFiles()				Delete any corrupt or empty data files created by the indexer
void vPurgeInvalidDataFiles(SsApMaint *poApp)
{
	Tchar szDirFilename[128], szNameFilename[128];
	Tchar szMessage[128];
  TMapMachine *psCurMachine;
	Tuint nCurMachine;
	FILE *pfDirFile, *pfNameFile;
	Tuint nDirSize, nNameSize;
	Tuint nDirID, nNameID;
	Tchar cNameChar, cDirChar;

	for(nCurMachine=0; nCurMachine<SS_CONTROL()->nGetNumValidMachines(); nCurMachine++)
	{
		psCurMachine = SS_CONTROL()->poGetValidMachine(nCurMachine);

		if(psCurMachine->m_szNetName[0] == 0)
		{
			continue;
		}

		sprintf(szNameFilename, "%s\\temp\\index\\%s.tmn", g_aszRootPath, psCurMachine->m_szNetName);
    sprintf(szDirFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szNetName);
		pfNameFile = fopen(szNameFilename, "rb");
		pfDirFile = fopen(szDirFilename, "rb");
		if(pfNameFile && pfDirFile)
		{
			nNameID		= _fileno(pfNameFile);
			nDirID 		= _fileno(pfDirFile);
			nNameSize = _filelength(nNameID);
			nDirSize	= _filelength(nDirID);

			if(nNameSize == 0 || nDirSize == 0)
			{
				fclose(pfNameFile);
				fclose(pfDirFile);
				DeleteFile(szNameFilename);
				DeleteFile(szDirFilename);
				sprintf(szMessage, "&R&*&)& Deleted empty datafile from &b&%s\n", psCurMachine->m_szNetName);
				poApp->vPrintMessage(szMessage);
				continue;
			}

			fseek(pfNameFile, -9, SEEK_END);		cNameChar = fgetc(pfNameFile);
			fseek(pfDirFile, -1, SEEK_END);			cDirChar = fgetc(pfDirFile);


			if(cNameChar != '>' || cDirChar != 0)
			{
				fclose(pfNameFile);
				fclose(pfDirFile);
				DeleteFile(szNameFilename);
				DeleteFile(szDirFilename);
				sprintf(szMessage, "&R&*&)& Purged corrupt datafiles from &b&%s&0& &g&(invalid format)\n", psCurMachine->m_szNetName);
				poApp->vPrintMessage(szMessage);
				continue;
			}

			fclose(pfNameFile);
			fclose(pfDirFile);
		}
		else
		{
			if(pfNameFile)		fclose(pfNameFile);
			if(pfDirFile)			fclose(pfDirFile);
		}
	}
}

// void vCreateSearchMachineDB()				Creates the SearchMachine Database containing only machines with entries
void vCreateSearchMachineDB(SsApMaint *poApp)
{
	Tchar szDirFilename[128], szSearchDbFilename[128], szFilename[128];
  TMapMachine *psCurMachine;
	TMachine sSearchMachine;
	Tuint nCurMachine, nCurDatabase;
	FILE *pfDirFile, *pfSearchDB, *pfMachineException;
	TMachineException *psMachineExceptions = NULL;
	Tuint nCurException;
	Tchar szExceptionBuffer[128];
	Tchar szMessage[128];

	// Read in exceptions
	sprintf(szFilename, "%s\\data\\MachineException.txt", g_aszRootPath);
	pfMachineException = fopen(szFilename, "r");
	if(pfMachineException)
	{
		fgets(szExceptionBuffer, 128, pfMachineException);
		nValidExceptions = atoi(szExceptionBuffer);
		psMachineExceptions = new TMachineException [nValidExceptions];

		for(nCurException=0; nCurException<nValidExceptions; nCurException++)
		{
			fgets(szExceptionBuffer, 128, pfMachineException);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psMachineExceptions[nCurException].m_szIpAddress, szExceptionBuffer);

			fgets(szExceptionBuffer, 128, pfMachineException);
			if(strlen(szExceptionBuffer) > 0)
				szExceptionBuffer[strlen(szExceptionBuffer)-1] = 0;
			strcpy(psMachineExceptions[nCurException].m_szBaseFtpURL, szExceptionBuffer);
		}	
		fclose(pfMachineException);
	}


	sprintf(szSearchDbFilename, "%s\\data\\SearchMachineDB.dat", g_aszRootPath);
	pfSearchDB = fopen(szSearchDbFilename, "wb");

	for(nCurMachine=0; nCurMachine<SS_CONTROL()->nGetNumValidMachines(); nCurMachine++)
	{
		psCurMachine = SS_CONTROL()->poGetValidMachine(nCurMachine);
		if(psCurMachine->m_szNetName[0] == 0)
		{
			continue;
		}

    sprintf(szDirFilename, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szNetName);
		pfDirFile = fopen(szDirFilename, "rb");
		if(pfDirFile)
		{
			fclose(pfDirFile);

			strcpy(sSearchMachine.m_szIpAddress, psCurMachine->m_szIpAddress);
			strcpy(sSearchMachine.m_szName, psCurMachine->m_szNetName);
			strcpy(sSearchMachine.m_szBaseFtpURL, "");
			sSearchMachine.m_nValidMachineIndex = psCurMachine->m_nValidMachineIndex;
			sSearchMachine.m_bMachineUp = psCurMachine->m_bMachineUp;
			sSearchMachine.m_bIsFTP = False;
			sSearchMachine.m_fFileBytes = 0.0f;
			sSearchMachine.m_nNumFiles = 0;
			for(nCurDatabase=0; nCurDatabase < NUMDATABASES; nCurDatabase++)
			{
				sSearchMachine.m_nNameOffset[nCurDatabase] 	= c_nInvalidPtr;
				sSearchMachine.m_nDirOffset[nCurDatabase] 	= c_nInvalidPtr;
				sSearchMachine.m_fDatabaseFileBytes[nCurDatabase] = 0.0f;
				sSearchMachine.m_nDatabaseNumFiles[nCurDatabase] = 0;
			}

			for(nCurException=0; nCurException<nValidExceptions; nCurException++)
			{
				if(strcmp(psMachineExceptions[nCurException].m_szIpAddress, psCurMachine->m_szIpAddress) == 0)
				{
					strcpy(sSearchMachine.m_szBaseFtpURL, psMachineExceptions[nCurException].m_szBaseFtpURL);
					sSearchMachine.m_bIsFTP = True;
					sprintf(szMessage, "  &b&*&0& Proccessed Exception %s -> %s\n", sSearchMachine.m_szName, sSearchMachine.m_szBaseFtpURL);
					poApp->vPrintMessage(szMessage);
					break;
				}
			}	

			fwrite((void *)&sSearchMachine, sizeof(TMachine), 1, pfSearchDB);
		}
	}
	fclose(pfSearchDB);

	if(psMachineExceptions)
	{
		delete [] psMachineExceptions;
	}
}

// void vPurgeInvalidDataFiles()				Compile all indexer data files into searchable databases
void vCompileVersion3(SsApMaint *poApp)
{
	Tuint nCurDatabase=0, nCurMachine=0;	
	TMachine *psCurMachine=NULL;
	Tchar szBuffer1[256], szBuffer2[256];
	FILE *pFiles[NUMDATABASES*2];
	FILE *pDirFile=0, *pNameFile=0;
	Tchar *pszDirMemory = (Tchar *)malloc(10*1024*1024);
	Tuint nDirFileSize=0, nNameFileSize=0;
	Tuint nLastDir[NUMDATABASES];
	fpos_t nFilePos=0;
	Tchar szMessage[1024];

	memset(szBuffer1, 0, 256);
	memset(szBuffer2, 0, 256);
	memset(pFiles, 0, sizeof(pFiles));
	memset(pszDirMemory, 0, 10*1024*1024);
	memset(nLastDir, 0, sizeof(nLastDir));
	memset(szMessage, 0, 1024);

  Sleep(5000);

  // Create Databases
	for(nCurDatabase = 0; nCurDatabase < NUMDATABASES; nCurDatabase++)
	{
		// Create new Name file for this database
		memset(szBuffer1, 0, sizeof(szBuffer1));			
		sprintf(szBuffer1, "%s\\data\\search\\%s", g_aszRootPath, g_aszFilenames[2*nCurDatabase]);	
		pFiles[2*nCurDatabase] = fopen(szBuffer1,"wb");
		

    // Create new Dir file for this database
		memset(szBuffer1, 0, sizeof(szBuffer1));			
		sprintf(szBuffer1, "%s\\data\\search\\%s", g_aszRootPath, g_aszFilenames[2*nCurDatabase+1]);	
		pFiles[2*nCurDatabase+1] = fopen(szBuffer1,"wb");
		ASSERT(pFiles[2*nCurDatabase+1] != NULL);	
	}

	for(nCurMachine=0; nCurMachine<SS_CONTROL()->nGetNumSearchMachines(); nCurMachine++)
	{
		psCurMachine = SS_CONTROL()->poGetSearchMachine(nCurMachine);

		sprintf(szMessage, "%s: ", psCurMachine->m_szName);
		poApp->vPrintMessage(szMessage);

		// Open Dir & Name indexer data file for this machine
		sprintf(szBuffer1, "%s\\temp\\index\\%s.tmd", g_aszRootPath, psCurMachine->m_szName);
		sprintf(szBuffer2, "%s\\temp\\index\\%s.tmn", g_aszRootPath, psCurMachine->m_szName);
		pDirFile = fopen(szBuffer1, "rb");
		pNameFile = fopen(szBuffer2, "rb");
		if(!pDirFile || !pNameFile)
		{
			if(pDirFile) fclose(pDirFile);
			if(pNameFile) fclose(pNameFile);
			continue;
		}

		nNameFileSize 	= _filelength(_fileno(pNameFile));
		nDirFileSize 		= _filelength(_fileno(pDirFile));
		if(!nNameFileSize || !nDirFileSize)
		{
			fclose(pDirFile);
			fclose(pNameFile);
			continue;
		}

		// Read entire Directory data file into memory
		nDirFileSize = _filelength(_fileno(pDirFile));
		fread((void *)pszDirMemory, nDirFileSize, 1, pDirFile);
		fclose(pDirFile);

		for(nCurDatabase = 0; nCurDatabase < NUMDATABASES; nCurDatabase++)
		{
				nLastDir[nCurDatabase] = c_nInvalidPtr;
				fgetpos(pFiles[2*nCurDatabase], &nFilePos);			// Get current position of BigDir file
				psCurMachine->m_nNameOffset[nCurDatabase] = (Tuint)nFilePos;			// Store pointer of Name Offset
				fgetpos(pFiles[2*nCurDatabase+1], &nFilePos);		// Get current position of BigName file
				psCurMachine->m_nDirOffset[nCurDatabase] = (Tuint)nFilePos;			// Store pointer of Dir offset
		}

		while(!feof(pNameFile))
		{
			Tuint nNameSize=0;
			Tuint nCurrentDirPtr=0;
			Tuint nNewDirStart=0;
			Tuint nDirPtr[NUMDATABASES];
			memset(nDirPtr, 0, sizeof(nDirPtr));

			memset(szBuffer1, 0, sizeof(szBuffer1));
			nNameSize = nReadName(szBuffer1, pNameFile);		// Read Filename and trailing Dir Ptr
			if(nNameSize==0)	break;												// Indicates EOF
			nCurrentDirPtr = *((int*)&szBuffer1[nNameSize-8]); // Get Dir Ptr

			// Figure out which database it belongs in
			nCurDatabase = nExtensionToFile(szBuffer1, nNameSize);

      // This may be the start of a new directory, so cache the file position
			fgetpos(pFiles[2*nCurDatabase], &nFilePos);
			nNewDirStart = (Tuint)nFilePos;

			if(nCurrentDirPtr > nLastDir[nCurDatabase] || nLastDir[nCurDatabase] == c_nInvalidPtr)
			{
				fgetpos(pFiles[2*nCurDatabase+1], (fpos_t *)&nFilePos);					//Get position for start of next directory name
				nDirPtr[nCurDatabase] = (Tuint)nFilePos;

				*((int*)&szBuffer1[nNameSize-8]) = nDirPtr[nCurDatabase];				//Set files DirPtr to end of DirFile

				fwrite((char*)szBuffer1, nNameSize, 1, pFiles[2*nCurDatabase]);	//write filename+(dirptr&filesize) to file

				int *size = (int *)&szBuffer1[nNameSize-4];
				psCurMachine->m_fFileBytes += (float)*size;
				psCurMachine->m_nNumFiles++;
				psCurMachine->m_fDatabaseFileBytes[nCurDatabase] += (float)*size;
				psCurMachine->m_nDatabaseNumFiles[nCurDatabase]++;

				nLastDir[nCurDatabase] = nCurrentDirPtr;	//keeps track of which directories we've copied
				Tuint nTemp = strlen(&pszDirMemory[nCurrentDirPtr]);
				fwrite((char*)&pszDirMemory[nCurrentDirPtr], strlen(&pszDirMemory[nCurrentDirPtr])+1 , 1 ,pFiles[2*nCurDatabase+1]); //write Directory to BIGDIR
				szBuffer2[0]='<';
				fwrite((void *)szBuffer2, 1, 1, pFiles[2*nCurDatabase+1]);
				fwrite((void *)&nNewDirStart, sizeof(Tuint), 1, pFiles[2*nCurDatabase+1]);	//ptr to start of new directory of files
			}
			else
			{
				*((int*)&szBuffer1[nNameSize-8]) = nDirPtr[nCurDatabase];			//Set files DirPtr to end of DirFile
				int *size = (int *)&szBuffer1[nNameSize-4];
				psCurMachine->m_fFileBytes += (float)*size;
				psCurMachine->m_nNumFiles++;
				psCurMachine->m_fDatabaseFileBytes[nCurDatabase] += (float)*size;
				psCurMachine->m_nDatabaseNumFiles[nCurDatabase]++;
				fwrite((void *)szBuffer1, nNameSize, 1, pFiles[2*nCurDatabase]);	//write filename+(dirptr&filesize) to file
			}
		}//end of while() no more (or none)filenames in this machine

		for(nCurDatabase = 0; nCurDatabase < NUMDATABASES; nCurDatabase++)
		{

			szBuffer2[0]='>';
			fgetpos(pFiles[2*nCurDatabase+1], (fpos_t*)&nFilePos);	//write '>' (end of machine/dir)
			if((Tuint)nFilePos>1)
			{	//write end of Machine marker '>' to Directory file
				fseek(pFiles[2*nCurDatabase+1], -5, SEEK_CUR);
				fwrite(szBuffer2, 1, 1, pFiles[2*nCurDatabase+1]);
				fseek(pFiles[2*nCurDatabase+1], 0, SEEK_END);
			}
			fgetpos(pFiles[2*nCurDatabase], (fpos_t*)&nFilePos);
			if((Tuint)nFilePos>1)
			{
				//cap filenames
				fseek(pFiles[2*nCurDatabase], -9, SEEK_CUR);
				fwrite(szBuffer2, 1, 1, pFiles[2*nCurDatabase]);
				fseek(pFiles[2*nCurDatabase], 0, SEEK_END);
			}
			if(nLastDir[nCurDatabase] == c_nInvalidPtr)	//IF no entries for a particular machines DB, set -1 ptr
			{
				psCurMachine->m_nNameOffset[nCurDatabase] = c_nInvalidPtr;
				psCurMachine->m_nDirOffset[nCurDatabase]  = c_nInvalidPtr;
			}
		}			
		fclose(pDirFile);
		fclose(pNameFile);		

		// Output stats
		float fSize = psCurMachine->m_fFileBytes / (1024.0f * 1024.0f);
		sprintf(szMessage, "%-.2f MB in %d files\n", fSize, psCurMachine->m_nNumFiles);
		poApp->vPrintMessage(szMessage);
	}
	free(pszDirMemory);
	pszDirMemory = NULL;

  // Close Databases
	for(nCurDatabase = 0; nCurDatabase < NUMDATABASES; nCurDatabase++)
	{
		fclose(pFiles[2*nCurDatabase]);
    fclose(pFiles[2*nCurDatabase+1]);
	}
}

STRANGEAPPTHREAD(StrangeAppThread_Maintenance)
{
	SsApMaint *poApp = (SsApMaint *)poStrangeApp;
	char szMessage[128];
	int ctr=0;
	poApp->m_bIsRunning = true;
	Tboolean bIsValid = False;
	CTime oTime = CTime::GetCurrentTime();
	CString oTimeString;

	OutputDebugString("---- Maintenance Started -----\n");
	if(!bShutdownAllScan())
	{
		poApp->vPrintMessage("&R&[ * * &0&Maintenance Service failed to terminate all scanning apps &R&* * ]\n");
		poApp->m_bIsRunning = false;
		poApp->m_bHaltCommandRequested = false;
		_endthread();
		return;
	}
	poApp->vPrintMessage("&R&*&0& Terminated all scanning applications\n");

	// Flush all maps
	SS_CONTROL()->vFlushAllMaps();
	poApp->vPrintMessage("&R&*&0& Flushed memory mapped files\n");

	// Process exception (Don't Scan, Scan as FTP, etc)
	poApp->vPrintMessage("&R&*&0* Processing exceptions...\n");
	vProcessMachineExceptions(poApp);

	// Purge corrupt or empty indexer data files
  poApp->vPrintMessage("&R&*&0* Searching for invalid indexer files...\n");
	vPurgeInvalidDataFiles(poApp);
	
 	// Create Search Machine Database
  poApp->vPrintMessage("&R&*&0* Generating Search Machine DB...\n");
  SS_CONTROL()->vCloseFileMapping(c_nSsStControlSearchMachineMap);
	vCreateSearchMachineDB(poApp);
	SS_CONTROL()->vOpenFileMapping(c_nSsStControlSearchMachineMap);

	// Compile Valid Indexer files into Searchable Databases
	SS_CONTROL()->vCloseFileMapping(c_nSsStControlDatabases);
  poApp->vPrintMessage("&R&*&0* Compiling Search 3.0 Databases...\n");
	vCompileVersion3(poApp);
	SS_CONTROL()->vOpenFileMapping(c_nSsStControlDatabases);

	// Output Top 10
	/*poApp->vPrintMessage("Top 10 Overall\n");
	poApp->vPrintMessage("---------------------------\n");
	for(int i=0; i<10; i++)
	{
		sprintf(szMessage, "%d: %s (%-.2f GB)\n", i+1, sGlobalStatData.nameMostFileBytes[i], sGlobalStatData.mostFileBytes[i] / (1024.0f * 1024.0f * 1024.0f));
		poApp->vPrintMessage(szMessage);
	}*/

	//poApp->vPrintMessage("\n");

	//vGenerateStatisticsPage(poApp);

	oTime = CTime::GetCurrentTime();
	oTimeString = oTime.Format("%I:%M%p on %A %B %d, %Y");
	sprintf(szMessage, "&R&* &b&Maintenance completed successfully&0& at %s\n", (LPCTSTR)oTimeString);
	poApp->vPrintMessage(szMessage);
  OutputDebugString("---- Maintenance Finished -----\n");
  gs_poSsControl->poGetStrangeControl()->bSearchingAllowed = 1;

	poApp->m_bIsRunning = false;
	poApp->m_bHaltCommandRequested = false;
	_endthread();
}

void SsApMaint::vBeginExecution()
{
	if(!m_bIsRunning)
	{
		RUN_STRANGEAPPTHREAD(StrangeAppThread_Maintenance);
	}
}


