/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
 
#ifndef _INC_SS_NU_PINGER
#define _INC_SS_NU_PINGER
#include <stdlib.h>
#include <afxwin.h>
#include <process.h>
#include <io.h>
#include <conio.h>
#include <time.h>
#include <winsock2.h>		
#include <nb30.h>			//LINK with netapi32.lib

typedef struct tagIPINFO
{
	u_char Ttl;				// Time To Live
	u_char Tos;				// Type Of Service
	u_char IPFlags;			// IP flags
	u_char OptSize;			// Size of options data
	u_char FAR *Options;	// Options data buffer
}IPINFO, *PIPINFO;

typedef struct tagICMPECHO
{
	u_long Source;			// Source address
	u_long Status;			// IP status
	u_long RTTime;			// Round trip time in milliseconds
	u_short DataSize;		// Reply data size
	u_short Reserved;		// Unknown
	void FAR *pData;		// Reply data buffer
	IPINFO	ipInfo;			// Reply options
}ICMPECHO, *PICMPECHO;


#define NUMNAMES 32		//maximum number of names for adapter
struct NBINFO
{
	ADAPTER_STATUS  adapt;
	NAME_BUFFER		NameBuff[NUMNAMES];
};

class SsNuPinger
{
public:
	SsNuPinger();
	~SsNuPinger();
	bool bRetrieveIP(char *Name, char *IP);			// gets IP from Netbios or Domain name
	bool bRetrieveName(char *Name, char *IP);		// gets fully qualified domain name
	bool bGetNetbiosName(char *Name, char *IP);	// Gets Netbios Name from IP
	bool bGetMAC(unsigned char *MAC, char *IP); // Gets MAC address from IP
	bool bPingByIP(char *IP);										// Ping an IP (returns true if responce)
	bool bPingByName(char *Name);								// Ping a Netbios or qualified Domain Name

private:
	bool   NameStartUp();
	void   NameShutDown();

	WSADATA wsaData;		// WSADATA
	ICMPECHO icmpEcho;		// ICMP Echo reply buffer
	HMODULE hndlIcmp;		// LoadLibrary() handle to ICMP.DLL
	// ICMP.DLL Export Function Pointers
	HANDLE (WINAPI *pIcmpCreateFile)(VOID);
	BOOL (WINAPI *pIcmpCloseHandle)(HANDLE);
	DWORD (WINAPI *pIcmpSendEcho)(HANDLE,DWORD,LPVOID,WORD,PIPINFO,LPVOID,DWORD,DWORD);
};


#endif
