/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "search35.h"
#include "rand.h"
#include <io.h>
#include <afxwin.h>
#include <time.h>
#include <assert.h>
#include <memory.h>
#include <stdlib.h>
#include <string.h>
#include "searchhtml.h"

#define CMDLINE_TEST 0

#define SEARCH_ASSERT(cond,code) { if(!(cond)) SafeCgiExit(code); }

void SafeCgiExit(int code);
void SafeCgiPrintError(char *errormsg, char *orig_arg);

Tuint RandomSeedUsed = time(NULL);

Tchar szResultLeadin[1024];
Tchar szResultFilename[1024];
Tchar szResultFoldername[1024];
Tchar szResultSize[1024];
Tchar szResultLeadout[1024];

TStrangeControl *psStrangeControl;

Tuint StrangeGetFileSize(Tchar *Filename)
{
	FILE *temp = fopen(Filename, "r");
	Tuint length = _filelength(_fileno(temp));
	fclose(temp);
	return length;
}

BanInfo banned[] = {
  {"0", -1}
};

SS3::SS3()
{
	int x;
	psAllMachines = NULL;
	nNumMachines = 0;
	NumFound=0;
	FoundBaseCount=1;
	OpenDataFiles();
	OpenMachineInfo();
	RandomRef=(int*)malloc(nNumMachines*sizeof(int));
	SEARCH_ASSERT(RandomRef!=NULL,0);
	for(x=0;x<nNumMachines;x++)
		RandomRef[x]=x;
	memset(&S,0,sizeof(pSrch));
	bProvideDirLinks = false;
	bReturnSmbLinks = false;
}

SS3::~SS3()
{
	CloseMachineInfo();
	CloseDataFiles();
	free(RandomRef);
}
// OpenDataFile
//Does:		Opens all databases
//Returns:	(bool) success failure
bool SS3::OpenDataFiles()
{
	int		x;
	Tchar Filename[1024];

  HANDLE hControlMap = OpenFileMapping(FILE_MAP_READ,TRUE,MAPNAME_STRANGECONTROLMAP);
  psStrangeControl = (TStrangeControl *)MapViewOfFile(hControlMap,FILE_MAP_READ,0,0,0);
	
  SEARCH_ASSERT(psStrangeControl, 1);
  SEARCH_ASSERT(psStrangeControl->bSearchingAllowed, 2);

	hSearchMonitorMap = OpenFileMapping(FILE_MAP_READ|FILE_MAP_WRITE,TRUE,MAPNAME_SEARCHMONITORMAP);
	SEARCH_ASSERT(hSearchMonitorMap, 3);
	psSearchInfoBase = (TSearchInfo *)MapViewOfFile(hSearchMonitorMap,FILE_MAP_READ|FILE_MAP_WRITE,0,0,0);

	for(x=0;x<NUMDATABASES;x++)
	{
		hHandles[2*x]=OpenFileMapping(FILE_MAP_READ,TRUE,g_aszMapNames[2*x]);
		if(hHandles[2*x]==NULL)
		{
			Name[x] = NULL;
		}
		else
		{
			Name[x]=(char*)MapViewOfFile(hHandles[2*x],FILE_MAP_READ,0,0,0);
			SEARCH_ASSERT(Name[x]!=NULL, 5);
		}
		hHandles[2*x+1]=OpenFileMapping(FILE_MAP_READ,TRUE,g_aszMapNames[2*x+1]);
		if(hHandles[2*x+1]==NULL)
		{
			Dir[x] = NULL;
		}
		else
		{
			Dir[x]=(char*)MapViewOfFile(hHandles[2*x+1],FILE_MAP_READ,0,0,0);
			SEARCH_ASSERT(Dir[x]!=NULL, 7);
		}

		sprintf(Filename, "%s\\data\\search\\%s", psStrangeControl->szRootPath, g_aszFilenames[2*x]);
		FileSize[x] = StrangeGetFileSize(Filename);
	}

	HtmlMap.OpenMap(MAPNAME_STRANGEHTML, True);
	Html = (char *)HtmlMap.GetPointer();
	sprintf(Filename, "%s\\data\\SearchTemplate.html", psStrangeControl->szRootPath);
	SEARCH_ASSERT(Html, 8)
	HtmlSize = StrangeGetFileSize(Filename);
	InitSearchHtml(Html, HtmlSize);
	return(true);

}

// OpenMachineInfo
//Does:		Opens Machine.dat and control.dat
//Returns:	(bool) success failure
bool SS3::OpenMachineInfo()
{
	Tint nFileSize;

	hMachineMap		= OpenFileMapping(FILE_MAP_READ | FILE_MAP_WRITE, TRUE, MAPNAME_SEARCHMACHINEMAP);
	psAllMachines	= (TMachine *)MapViewOfFile(hMachineMap, FILE_MAP_READ | FILE_MAP_WRITE, 0, 0, 0);

	Tchar Filename[1024];
	sprintf(Filename, "%s\\data\\SearchMachineDB.dat", psStrangeControl->szRootPath);
	nFileSize = StrangeGetFileSize(Filename);
	nNumMachines = nFileSize / sizeof(TMachine);
	return(true);
}

bool SS3::CloseDataFiles()
{
	int x;
	for(x=0;x<NUMDATABASES;x++)
	{
		UnmapViewOfFile(Name[x]);
		CloseHandle(hHandles[2*x]);
		UnmapViewOfFile(Dir[x]);
		CloseHandle(hHandles[2*x+1]);
	}

	UnmapViewOfFile(psSearchInfoBase);
	CloseHandle(hSearchMonitorMap);

	return(true);
}
bool SS3::CloseMachineInfo()
{
	UnmapViewOfFile(psAllMachines);
	CloseHandle(hMachineMap);
	return(true);
}
// Prefix
//Does:		Creats KMP Prefix
//Inputs:	(char *Pi) place constructed prefix here(out)
//			(char *P1) String to create prefix for(in)
//Returns:	(void)
void SS3::Prefix(char *Pi, char *P1)
{
	int q,k,m;
	m=strlen(P1);
	Pi[0]=0; 
	k=0;
	for(q=1;q<m;q++) 
	{
		while(k>0 && P1[k] != P1[q])
			k=Pi[k];
		if(P1[k]==P1[q]) 
			k++;
		Pi[q]=k;
	}
}

// SkipToNextDirectory
//Inputs:	
//			(int Hint) Where I THINK the end of the filename is *Thus where the directory ptr is* (in)
//Global:	(pSrch S) Search Info (in/out)
//Does:		Takes Hint and updates SrchInfo->pDir and ->pName to NEXT Directory
//Returns:	(bool) Success/failure (failure is no more directories on this machine
bool SS3::SkipToNextDirectory(int Hint)	//Gets/sets to first file in next directory
{
	int Start;
	if(Dir[S.DB][Hint]=='>')	//if this was the last dir on this machine
		return(false);
	Hint+=9;
	Start=Hint;
	while(1)
	{
		if(Dir[S.DB][Hint]==0)  //  '/0'  (found end of the directory string, pointer follows)
		{
			S.pStart=*((int*)&Dir[S.DB][Hint+2]);
			S.pDir=Start;
			return(true);
		}
		else
			Hint++;
	}
}

// GetNextDir
//Inputs:	(void)
//Global:	(pSrch S) Search Info (in/out)
//Does:		Uses the SrchInfo->pName and sets the SrchInfo->pDir to the Directory
//Returns:	(bool) SUccess/failure
bool SS3::GetNextDir()
{
	int Counter=0;
	int Temp=S.pStart;
	while(Counter<270)
	{
		if(Name[S.DB][Temp]=='<')			//check for end of word
		{
			S.pDir=*((int*)&Name[S.DB][Temp+1]);
			return(true);					//return pointer to directory string
		}
		else if(Name[S.DB][Temp]=='>')	//check for end of machine
		{
			//return(false);
			S.pDir=*((int*)&Name[S.DB][Temp+1]);
			return(true);					//return pointer to directory string
		}
		Temp++;						//just another character,increment
		Counter++;
	}
	SEARCH_ASSERT(0,10);
	return(false);
}


// DirMatch
//Inputs:	(char* Text) Text string to search (in)
//			(char **Strings) List of Querry Strings to look for (in)
//			(int *Sizes) Array of Strlen(Strings[x])	(in)
//			(int NumTerms) Number of Querry Strings (in)
//			(int *pEnd) This is the Index to the end of the filename (in/out) *used later in program to read filesize and directory ptr*
//			(ulong *Mask) Records which terms pass(HIT/ANTIMISS) or fail (ANTI/MISS)(in\out)
//Does:		Does String matching on Directory portion of [dir][filename]
//Returns:	(int) HIT(all terms match) MISS(some or none match) ANTIHIT(there was an anti match)
//Notes:    *pEnd must be set to the ABSOLUTE INDEX of the start of Text..so that it can add relative offset
int SS3::DirMatch(char *Text, char **Strings, int *Sizes, int NumTerms, int *pEnd, unsigned long *Mask)
{
	int		Counter;	//counts which string we are matching
	int		Match;		//counts number of matches
	int		i,q;
	int		Modifier;	
	int		pFront;
	bool	Anti;	//matching an (-)
	bool    Exact;	//Exact match 
//NOTE: Get rid of excessive array notation 
	Counter=0;
	Match=0;
	*Mask=0;
	while(Counter<NumTerms)
	{
		if(Strings[Counter]==NULL)
		{
			Counter++;
			continue;
		}
		i=0;
		pFront=0;
		Anti=false;
		Exact=false;
		if(Strings[Counter][0]=='-')
		{
			Anti=true;
			pFront=1;
			if(Strings[Counter][1]=='\"') //Exact Match
			{
				Exact=true;
				pFront=2;
			}
		}
		if(Strings[Counter][0]=='\"')	
		{
			pFront=1;
			Exact=true;
		}
		q=pFront;
		while(1)
		{
			if(Text[i]>96 && Text[i]<123)		//compensate for upper or lower case
				Modifier=-32;
			else
				Modifier=0;

			if(Strings[Counter][q]==(Text[i]+Modifier))	//character match
				q++;
			else
				q=pFront;	//This is where a prefix may go
			if(q==Sizes[Counter])				//hit on this term
			{
				if(Anti)
				{
					if(Exact)
					{
						if((Text[i-Sizes[Counter]+pFront]==32 || Text[i-Sizes[Counter]+pFront]=='\\' || i-(Sizes[Counter]-(pFront+1))==0) && 
						  (Text[i+1]==32 || Text[i+1]=='\\' || Text[i+1]==0))	
						{	
							//exact match
							(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=ANTIHIT;
							while(Text[i]!='<' && Text[i]!='>')
								i++;
							*pEnd+=i;
							return(ANTIHIT);
						}
						else
						{
							//not exact match
							i++;
							q=pFront;
							continue;
						}
					}
					else
					{
						(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=ANTIHIT;
						while(Text[i]!='<' && Text[i]!='>')
							i++;
						*pEnd+=i;
						return(ANTIHIT);
					}
				}
				if(Exact)
				{
					if((Text[i-Sizes[Counter]+pFront]==32 || Text[i-Sizes[Counter]+pFront]=='\\' || i-(Sizes[Counter]-(pFront+1))==0) && 
					  (Text[i+1]==32 || Text[i+1]=='\\' || Text[i+1]==0))	
					{
						//exact match
						(*Mask)|=(0x00000001<<Counter);  //Matches[Counter]=HIT;
						Match++;
						Counter++;	
					}
					else
					{
						//not exact match
						i++;
						q=pFront;
						continue;
					}
				}
				else
				{
					(*Mask)|=(0x00000001<<Counter);  //Matches[Counter]=HIT;
					Match++;
					Counter++;
				}
				break;
			}
			if(Text[i]=='<' || Text[i]=='>')	//miss on this term
			{
				if(Anti)
					(*Mask)|=(0x00000001<<Counter);//Matches[Counter]=ANTIMISS;
				else
					(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=MISS;
				Counter++;
				break;
			}
			i++;
		}		
	}
	while(Text[i]!='<' && Text[i]!='>')
		i++;

	*pEnd+=i;
	if( (~(0xFFFFFFFF<<NumTerms))==(*Mask))
		return(HIT);
	return(MISS);
}

// NameMatch
//Inputs:	(char* Text) Text string to search (in)
//			(char **Strings) List of Querry Strings to look for (in)
//			(int *Sizes) Array of Strlen(Strings[x])	(in)
//			(int NumTerms) Number of Querry Strings (in)
//			(int *pEnd) This is the Index to the end of the filename (in/out) *used later in program to read filesize and directory ptr*
//			(ulong mask) Records which terms pass(HIT/ANTIMISS) or fail (ANTI/MISS)(in\out)
//			(bool AntiMode) Just checking for (-)Anti querries (because Directory is already a hit   (in)
//Does:		Does String matching on filename portion of [dir][filename]
//Returns:	(int) HIT MISS ANTI
//Notes:    *pEnd must be set to the ABSOLUTE INDEX of the start of Text..so that it can add relative offset
int SS3::NameMatch(char *Text, char **Strings, int *Sizes, int NumTerms, int *pEnd, unsigned long *Mask)
{
	int		Counter;	//counts which string we are matching
	int		Match;		//counts number of matches
	int		i,q;
	int		Modifier;	
	int		pFront;		//points to front of querry string ([1] if (-) else [0]
	bool	Anti;		
	bool    Exact;
//NOTE: Get rid of excessive array notation 
	Counter=0;
	Match=0;
	*Mask=0;
	while(Counter<NumTerms)
	{
		if(Strings[Counter]==NULL)
		{
			Counter++;
			continue;
		}
			//MISS 
		i=0;
		pFront=0;
		Anti=false;
		Exact=false;
		if((*Mask)&(0x00000001<<Counter))
		{
			Counter++;
			continue;
		}
		if(Strings[Counter][0]=='-')
		{
			Anti=true;
			pFront=1;
			if(Strings[Counter][1]=='\"' && Strings[Counter][Sizes[Counter]]=='\"') //Exact Match
			{
				
				Exact=true;
				pFront=2;
			}
		}
		if(Strings[Counter][0]=='\"' && Strings[Counter][Sizes[Counter]]=='\"')	
		{
			pFront=1;
			Exact=true;
		}
		q=pFront;
		while(1)
		{
			if(Text[i]>96 && Text[i]<123)		//compensate for upper or lower case
				Modifier=-32;
			else
				Modifier=0;

			if(Strings[Counter][q]==(Text[i]+Modifier))	//character match
				q++;
			else
				q=pFront;	//This is where a prefix may go
			if(q==Sizes[Counter])		//MATCH!
			{
				if(Anti)
				{
					if(Exact)
					{
						if((Text[i-Sizes[Counter]+pFront]==32 || i-(Sizes[Counter]-(pFront+1))==0) && 
						   (Text[i+1]==32 || Text[i+1]=='\\' || Text[i+1]==0 || Text[i+1]=='.'))
						{	
							//exact match
							(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=ANTIHIT;
							while(Text[i]!='<' && Text[i]!='>')
								i++;
							*pEnd+=i;
							return(ANTIHIT);
						}
						else
						{
							//not exact match
							i++;
							q=pFront;
							continue;
						}
					}
					else
					{
						(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=ANTIHIT;
						while(Text[i]!='<' && Text[i]!='>')
							i++;
						*pEnd+=i;
						return(ANTIHIT);
					}
				}
				if(Exact)
				{
					if((Text[i-Sizes[Counter]+pFront]==32 || i-(Sizes[Counter]-(pFront+1))==0) && 
					   (Text[i+1]==32 || Text[i+1]=='\\' || Text[i+1]==0 || Text[i+1]=='.'))
					{
						//exact match
						(*Mask)|=(0x00000001<<Counter);  //Matches[Counter]=HIT;
						Match++;
						Counter++;	
					}
					else
					{
						//not exact match
						i++;
						q=pFront;
						continue;
					}
				}
				else
				{
					(*Mask)|=(0x00000001<<Counter);  //Matches[Counter]=HIT;
					Match++;
					Counter++;
				}
				break;
			}
			if(Text[i]=='<' || Text[i]=='>')	//NO MATCH!
			{
				if(Anti)
				{
					(*Mask)|=(0x00000001<<Counter); //Matches[Counter]=ANTIMISS;
					Counter++;
					break;
				}
				(*Mask)&=(~(0x00000001<<Counter)); //Matches[Counter]=MISS;
				*pEnd+=i;
				return(MISS);
			}
			i++;
		}
	}
	while(Text[i]!='<' && Text[i]!='>')
		i++;
	*pEnd+=i;

	return(HIT);
}

// Prioritize
//Inputs:	(char* List) List of QUerry Strings  (in/out)
//			(int *Sizes) Array of Strlen(List[x] (in/out)
//			(int NumTerms) (in)
//Does:		Changes order of querry List to get faster searches
//Returns:	(void)
void SS3::Prioritize(char **List, int *Sizes, int NumTerms)
{
	int x,y;
	int  iTemp;
	char *cTemp;

	for(x=0;x<NumTerms;x++)		//Bubble sort on Strlen (Long strings first..OR short (-) strings
	{
		for(y=0;y<NumTerms;y++)
		{
			if( (Sizes[x]>Sizes[y] && *List[x]!='-') ||
				(Sizes[x]<=Sizes[y] && *List[x]=='-')
			  )
			{
				iTemp=Sizes[x];				//swap [x] and [y]
				Sizes[x]=Sizes[y];
				Sizes[y]=iTemp;
				cTemp=List[x];
				List[x]=List[y];
				List[y]=cTemp;
			}
		}
	}
}

//Function:	LowAI
//
//Inputs:	(char** List) is List of querry strings, 
//			(int *Sizes) is strlen(List[x])
//			(int *NumTerms) is pointer to number of strings in List
//Does:		Manipulates querry to get better searches
//Returns:	(void)
void SS3::LowAI(char **List,int *Sizes,int *NumTerms)
{
	
}
//Function:	SetSearchOrder
//
//Inputs:	(void)
//Returns:	(void)
//Does:		Sets the order in which machines are searched
void SS3::SetSearchOrder()
{
	int x;
	int Swap;
	int Temp;
	TheRand.SeedRand(SEED);
	for(x=0;x<nNumMachines;x++)
	{
		Swap=TheRand.GetRand()%(nNumMachines);
		Temp=RandomRef[x];
		RandomRef[x]=RandomRef[Swap];
		RandomRef[Swap]=Temp;
	}
}

//Function:	Search
//
//Inputs:	(char* arg) string submitted by web page
//Returns:	(int) number of Hits (up to MAXHITS)
//Does:		Takes web string, and does all things needed to search and display web results
int SS3::Search(char *arg)
{
	int		NumTerms;			//number of querry terms
	char	*List[MAXTERMS];	//list of querry's
	int		Sizes[MAXTERMS];	//strlen(List[x])
	int   MinSize=0;			//File Must be bigger(in bytes) than this to hit
	int		x;

	if(nNumMachines == 0)
	{
		return 0;
	}

	if(!PreSearch(arg, &NumTerms, List))
		return(0);

	if(Name[S.DB] == NULL)
	{
		return(0);
	}

	switch(S.MinSizeLevel)	// Check out minimum size
	{
		case 1: MinSize = 1024;				break; // 1 Kilobyte
		case 2: MinSize = 1048576;		break; // 1 Megabyte
		case 3: MinSize = 5242880;		break; // 5 Megabytes
		case 4: MinSize = 10485760;		break; // 10 Megabytes
		case 5:	MinSize = 52428800;		break; // 50 Megabytes
		case 6: MinSize = 104857600;	break; // 100 Megabytes
		default : MinSize = 0;
	}

	for(x=0;x<NumTerms;x++)
	{
		Sizes[x]=strlen(List[x]);
		if(*List[x]=='\"' || *(List[x]+1)=='\"')	//probable bug here
			Sizes[x]--;
	}
	RemoveIllegal(List,Sizes,&NumTerms);

//	LowAI(List,Sizes,&NumTerms);
	Prioritize(List,Sizes,NumTerms);
	//Prefix()
	if(NumTerms==0)
		return(0);
	NumFound=NewSearch3(List,Sizes,NumTerms,MinSize);
	return(NumFound);
}

//returns true if there is a quoted passage in the string
bool SS3::DetectQuote(char* String)
{
	char *Ptr=String;
	if(*Ptr!='\"')
		return(false);
	Ptr++;
	while(1)
	{
		if(*Ptr==0)
			return(false);
		if(*Ptr=='\"')
		{
			while(Ptr!=String)
			{
				if(*Ptr=='+')
					*Ptr=32;
				Ptr--;
			}
			return(true);
		}
		Ptr++;
	}
	return(false);
}

//Function:	PreSearch
//
//Inputs:	(char* arg) the web submitted querry string (in)
//			(int *NumTerms) pointer to number of terms in querry string (out)
//			(char **List) List of querry strings (out)
//Global:	(pSrch S) Search Info (in/out)
//Does:		Parses input string, creates querry List, and sets up SearchInfo
//Returns:	(bool) success/fail
bool SS3::PreSearch(char *arg, int *NumTerms, char **List)
{
	char *Token;
	char *Temp;

	printf("Content-type: text/html\n\n");

    if(arg[0]!='D')
      return(false);

    Token=(char*)strtok(arg,"&");    
    if(Token==NULL)
      return(false);
	S.DB=atoi(&Token[2]);
	if(S.DB<0 || S.DB>NUMDATABASES)	//Bounds check databases
		return(false);
    
    Token=(char*)strtok(NULL,"&");
    if(Token==NULL)
		return(false);		//Bad string
	if(Token[0]!='A')
		return(false);
	if(strlen(Token)==2)
		return(false);
	Temp=(char*)Token;	//Save ptr to querry


	Token=(char*)strtok(NULL,"&");
	if(Token!=NULL && Token[0]=='C')       //skip this
		Token=(char*)strtok(NULL,"&");

	if(Token!=NULL && Token[0]=='S')			// process size directive
	{
		S.MinSizeLevel = atoi(&Token[2]);
		Token=(char*)strtok(NULL,"&");
	}
	if(Token==NULL)
	{
		//SEED=time(NULL);
		SEED=RandomSeedUsed;
		RandomSeedUsed = SEED;
		SetSearchOrder();
		S.MachineIndex=0;
		S.pStart=psAllMachines[RandomRef[S.MachineIndex]].m_nNameOffset[S.DB];
		if(S.pStart== c_nInvalidPtr)
		{
			if(!GetNextMachine())
				return(false);
		}
		SEARCH_ASSERT(S.pStart!=c_nInvalidPtr,11);
		S.pDir=psAllMachines[RandomRef[S.MachineIndex]].m_nDirOffset[S.DB];
		FoundBaseCount=1;
	}
	else
	{
		if(Token[0]!='E')	//pStart
			return(false);
		S.pStart=atoi(&Token[2]);
			
		Token=(char*)strtok(NULL,"&");
		if(Token==NULL)
			return(false);
		if(Token[0]!='F')	//Machine Index
			return(false);
		S.MachineIndex=atoi(&Token[2]);

		Token=(char*)strtok(NULL,"&");
		if(Token==NULL)
			return(false);
		if(Token[0]!='G')	//Counter
			return(false);
		FoundBaseCount=atoi(&Token[2]);

		Token=(char*)strtok(NULL,"&");
		if(Token==NULL)
			return(false);
		if(Token[0]!='H')	//SEED
			return(false);
		SEED=atoi(&Token[2]);
		SetSearchOrder();
		GetNextDir();		//Set pDir
	}
	
	
//BOUNDS CHECK!!!!!
	if(S.pStart<0 || S.pStart>((int)FileSize[S.DB]))
		return(false);
	if(S.MachineIndex<0 || S.MachineIndex>nNumMachines)
		return(false);
//need to ASSERT that if string is '-' string is atleast 2 chars 

	ChangeSpecial(Temp);
	*NumTerms=0;
	Temp+=2;
	DetectQuote(Temp);
	Token=(char*)strtok(Temp,"+");
	if(Token!=NULL)
	{
		List[*NumTerms]=Token;
		while(Token!=NULL)
		{
			DetectQuote(Temp+strlen(Token)+1);
			Token=(char*)strtok(NULL,"+");
			if(Token!=NULL)
			{
				(*NumTerms)++;
				List[*NumTerms]=Token;
			}
			if(*NumTerms>MAXTERMS)
				return(false);
		}
	}
	(*NumTerms)++;

	return(true);
}

int nHexCharToDigit(char ch)
{
  switch(ch)
  {
    case '0': case '1': case '2': case '3': case '4': case '5': case '6':
    case '7': case '8': case '9': 
      return ch-'0';

    case 'A': case 'B': case 'C': case 'D': case 'E': case 'F':
      return ch-'A'+10;

    default:
      return -1;
  }
}

//Function:	ChangeSpecial
//
//Inputs:	
//			(char *String) List of querry strings (in/out)
//Does:		Replaces any %22 with "  (put all %codes here)
//Returns:	(void)


void SS3::ChangeSpecial(char *String)
{
	int z,x;
	char Buffer[2048];
	memset(Buffer,0,2048*sizeof(char));
	SEARCH_ASSERT(strlen(String)<2048,14);
	x=0;z=0;
	while(String[x]!=0)
	{
		if(String[x]=='%' && String[x+1]=='2' && String[x+2]=='2')
		{
			Buffer[z]='\"';
			x+=3;
		}
		else
		{
      if(String[x]=='%' && String[x+1]!=0 && String[x+2]!=0)
      {
        int msd = nHexCharToDigit(String[x+1]);
        int lsd = nHexCharToDigit(String[x+2]);

        if(msd != -1 && lsd != -1)
        {
          Buffer[z] = msd*16+lsd;
          x+=3;
        }
      }
      else
      {
			  Buffer[z]=String[x];
			  x++;
      }
		}
		z++;
	}
	strcpy(String,Buffer);
}

//Function:	NewSearch3
//
//Inputs:	
//			(char **Strings) List of querry strings (in)
//			(int *Sizes) Array of Strlen(List[x]) (in)
//			(int NumTerms) Number of Querry Strings (in)
//Global:	(pSrch S) Search Info (in/out)
//Note:		(FoundFile SS3.F[]) Results placed here (out)
//Does:		This does the actual searching/string matching
//Returns:	(int) Number of hits (up to MAXHITS)
int SS3::NewSearch3(char **Strings, int *Sizes, int NumTerms, int MinSize)
{
	int		NumHits=0;
	int		OldiDir=c_nInvalidPtr;	//this is the pointer to the last used Directory
	int		NewiDir=0;	//this is the pointer to the next directory (or same)
	int 	iDirMatch;	//results of string matching against the DIrectory
	int     iNameMatch;	//results of string matching against the Name
	int		pDirEnd;	//Index to end of directory string	(end is first '<' or '>')
	int		pNameEnd;	//Index to end of Name String
	unsigned long NameMask;	//records the matches on the Name
	unsigned long DirMask;	//records the matches on the directory

	//SEARCH_ASSERT(S.MachineIndex<nNumMachines);
	pDirEnd=c_nInvalidPtr;
	pNameEnd=c_nInvalidPtr;
	pDirEnd=S.pDir;
	iDirMatch=DirMatch(&Dir[S.DB][S.pDir], Strings, Sizes, NumTerms, &pDirEnd,&DirMask);
	while(iDirMatch==ANTIHIT) 
	{
		if(SkipToNextDirectory(pDirEnd)==false) //If no more directories in this machine
		{
			if(!GetNextMachine()) //Go to Next Valid Machine
				return(NumHits); //no more machines
		}
		pDirEnd=S.pDir;
		iDirMatch=DirMatch(&Dir[S.DB][S.pDir],Strings, Sizes, NumTerms,&pDirEnd,&DirMask);
	}
	OldiDir=NewiDir=S.pDir;	//Old Directory String Pointer now points to new Directory Directory String
//now at a non anti Directory and matched.
	while(NumHits<MAXHITS && S.MachineIndex<nNumMachines) //till EOF or FULL Hit count
	{	
		pNameEnd=S.pStart;
//		if(iDirMatch==HIT)	//can skip
//		{
//			memcpy(&F[NumHits].DirName,&Dir[S.DB][S.pDir],pDirEnd-S.pDir);
//			memcpy(&F[NumHits].FileName,&Name[S.DB][S.pStart],pNameEnd-S.pStart);
//			F[NumHits].FileSize=*((int*)&Name[S.DB][pNameEnd+5]);
//			NumHits++;
//		}
		
		iNameMatch=NameMatch(&Name[S.DB][S.pStart],Strings, Sizes, NumTerms,&pNameEnd,&NameMask);  // Full Match against fileName

		NewiDir=*((int*)&Name[S.DB][pNameEnd+1]);	//get directory pointer at end of name we just matched
		if(iNameMatch==ANTIHIT)	//found an anti match
		{
			if(NewiDir!=OldiDir) //directory string pointer is different than expected
			{				
				do//find next valid directory pointer
				{
					if(SkipToNextDirectory(pDirEnd)==false) //If no more directories in this machine
					{
						if(!GetNextMachine()) //Go to Next Valid Machine
							return(NumHits); //no more machines to search..return results
					}
					pDirEnd=S.pDir;
					iDirMatch=DirMatch(&Dir[S.DB][S.pDir],Strings, Sizes, NumTerms,&pDirEnd,&DirMask);
				} while(iDirMatch==ANTIHIT);
				OldiDir=NewiDir=S.pDir;
			}
			if(Name[S.DB][pNameEnd]=='>') //If filename was last on the machine
			{
				//end of Machine..go to next
				if(!GetNextMachine())
					return(NumHits); //NO more machiens to search..return results
			}
			else
				S.pStart=pNameEnd+9;   // increment to next name
			continue;
		}//Name is matched dir is matched
		else if(NewiDir==OldiDir) //FileName is in same dir as was previously matched
		{
			if( (NameMask|DirMask) == (~(0xFFFFFFFF<<NumTerms)) )
			{
				F[NumHits].FileSize=*((int*)&Name[S.DB][pNameEnd+5]);
				if(F[NumHits].FileSize>MinSize)
				{
					memcpy(&F[NumHits].DirName,&Dir[S.DB][S.pDir],pDirEnd-S.pDir);
					memcpy(&F[NumHits].FileName,&Name[S.DB][S.pStart],pNameEnd-S.pStart);
					F[NumHits].MachineID=RandomRef[S.MachineIndex];
					NumHits++;
				}
			}
			//Go to Next Name
			if(Name[S.DB][pNameEnd]=='>') //If filename was last on the machine
			{
				//end of Machine..go to next
				if(!GetNextMachine())
					return(NumHits); //NO more machiens to search..return results
			}
			else
				S.pStart=pNameEnd+9;   // increment to next name
		}
		else //FileName is in a new dir..need to rematch
		{
			S.pDir=OldiDir=NewiDir;
			pDirEnd=S.pDir;
			iDirMatch=DirMatch(&Dir[S.DB][S.pDir],Strings, Sizes, NumTerms,&pDirEnd,&DirMask);
			if(iDirMatch==ANTIHIT)
			{
				do//find next valid directory pointer
				{
					if(SkipToNextDirectory(pDirEnd)==false) //If no more directories in this machine
					{
						if(!GetNextMachine()) //Go to Next Valid Machine
							return(NumHits); //no more machines to search..return results
					}
					pDirEnd=S.pDir;
					iDirMatch=DirMatch(&Dir[S.DB][S.pDir],Strings, Sizes, NumTerms,&pDirEnd,&DirMask);
				} while(iDirMatch==ANTIHIT);
				OldiDir=NewiDir=S.pDir;
				continue;
			}
			else if((NameMask|DirMask) == (~(0xFFFFFFFF<<NumTerms)))
			{
				F[NumHits].FileSize=*((int*)&Name[S.DB][pNameEnd+5]);
				if(F[NumHits].FileSize>MinSize)
				{
					memcpy(&F[NumHits].DirName,&Dir[S.DB][S.pDir],pDirEnd-S.pDir);
					memcpy(&F[NumHits].FileName,&Name[S.DB][S.pStart],pNameEnd-S.pStart);
					F[NumHits].MachineID=RandomRef[S.MachineIndex];
					NumHits++;
				}
			}
			//Go to Next Name
			if(Name[S.DB][pNameEnd]=='>') //If filename was last on the machine
			{
				//end of Machine..go to next
				if(!GetNextMachine())
					return(NumHits); //NO more machiens to search..return results
			}
			else
				S.pStart=pNameEnd+9;   // increment to next name
		}
	}//end while(1) (FULL hits)
	return(NumHits);
}

//Function:	GetNextMachine
//
//Inputs:	(void)
//Global:	(pSrch S) Search Info (in/out)
//Note:		Updates Searchinfo->pDir to next(or current) Directory
//Returns:	(bool) success/fail
bool SS3::GetNextMachine()
{
	// Look for Machines that are up, and we have indexed data for
	
	do{S.MachineIndex++;}
		while(S.MachineIndex<nNumMachines &&
			 (psAllMachines[RandomRef[S.MachineIndex]].m_bMachineUp==False   || 
			  psAllMachines[RandomRef[S.MachineIndex]].m_mScanFlags==1 || 
			  psAllMachines[RandomRef[S.MachineIndex]].m_nNameOffset[S.DB]==c_nInvalidPtr)); //Check for good machine


	if(S.MachineIndex==nNumMachines) //searched to end of Machine.dat
		{
			S.MachineIndex=nNumMachines; 
			S.pStart=c_nInvalidPtr;
			return(false); //NO MORE MACHINES TO SEARCH
		}
	S.pStart=psAllMachines[RandomRef[S.MachineIndex]].m_nNameOffset[S.DB];
	S.pDir=psAllMachines[RandomRef[S.MachineIndex]].m_nDirOffset[S.DB];
	return(true); //check this machine
}

void SS3::OutputHTML(char *passed_arg, char *orig_arg)
{
	char dirlink_info[2048];

	dirlink_info[0] = 0;

	HTML_PrintHeader(orig_arg, 2);

	for(int j=0;j<NumFound;j++)
		HTML_PrintResult(j+FoundBaseCount, &F[j]);
	
	printf("</TABLE>\n");

	if(NumFound == MAXHITS) 
	{
		HTML_Show_Nextlink(orig_arg, S.DB, S.pStart, S.MachineIndex, NumFound+FoundBaseCount, SEED);
	}
	else
	{
		PrintUntilToken("<!--searchcontinue");
		ReadLine(); // Search Continue Body
		ReadLine(); // Search Continue trailing
		ReadLine(); // <!--searchdone>
		ReadLine(); // Search Done body
		printf(GetCurrentLine(), NumFound+FoundBaseCount-1);
		ReadLine();
	}

	PrintUntilEnd();
}

void SS3::HTML_PrintHeader(char *searchterm, int database)
{
	char revised[2048];

	// Make sure search term is JUST the 'a=' portion
	char *p = searchterm;
	char *q = revised;

	*q = 0;

	while((*p != 0) && (*p != 'A') && (*p != 'a'))
	{
		// skip up to A/a character (if any)
		p++;
	}
	if(*p != 0)
	{
		p++;
		if(*p == '=')
		{
			p++;
			while((*p != 0) && (*p != '&'))
			{
				switch(*p)
				{
					case '+':	
						*q = ' ';
						break;
					default:
						*q = *p;
						break;
				}
				q++;
				p++;
				*q = 0;
			}
		}
	}

  ChangeSpecial(revised);

	database = S.DB;
	int num;
	PrintUntilToken("<!--searchdb");
	sscanf(GetCurrentLine(), "<!--searchdb:%d>", &num);
	for(Tint i=0; i<num; i++)
	{
		ReadLine();
		printf(GetCurrentLine(), (database == i) ? " selected" : "");
	}
	ReadLine(); // Skip </searchdb-->
	PrintUntilToken("<!--searchtext");
	ReadLine();
	printf(GetCurrentLine(), revised);
	ReadLine(); // Skip </searchtext-->

	PrintUntilToken("<!--searchopt");
	sscanf(GetCurrentLine(), "<!--searchopt:%d>", &num);
	for(i=0; i<num; i++)
	{
		ReadLine();
		printf(GetCurrentLine(), (S.MinSizeLevel == i) ? " selected" : "");
	}
	ReadLine(); // Skip </searchopt-->

	PrintUntilToken("<!--searchheader>");
	ReadLine();
	printf(GetCurrentLine(), "Filename");
	ReadLine();
	printf(GetCurrentLine(), bProvideDirLinks ? "Folder (Click to Browse)" : "Folder");
	ReadLine();
	printf(GetCurrentLine(), "Size");
	ReadLine(); // Skip </searchheader-->
	PrintUntilToken("<!--searchresult>");
	ReadLine();
	strncpy(szResultLeadin, GetCurrentLine(), 1024);
	ReadLine();
	strncpy(szResultFilename, GetCurrentLine(), 1024);
	ReadLine();
	strncpy(szResultFoldername, GetCurrentLine(), 1024);
	ReadLine();
	strncpy(szResultSize, GetCurrentLine(), 1024);
	ReadLine();
	strncpy(szResultLeadout, GetCurrentLine(), 1024);
	ReadLine(); // Skip </searchresult-->
	
}
	
void SS3::HTML_PrintResult(int j, FoundFile *F)
{
	float size;

	char szFilenameURL[512], szFolderURL[512], szFileSize[256], szDirText[256];
	char *p;

	char szFilenameAndLink[1024];
	char szDirnameAndLink[1024];

	if(psAllMachines[F->MachineID].m_bIsFTP)
	{
		sprintf(szFilenameURL, "\"%s%s%s\"", psAllMachines[F->MachineID].m_szBaseFtpURL, F->DirName, F->FileName);
		sprintf(szFolderURL, "\"%s%s\"", psAllMachines[F->MachineID].m_szBaseFtpURL, F->DirName);
		sprintf(szDirText, "FTP: %s%s", psAllMachines[F->MachineID].m_szName, F->DirName);
	}
	else
	{
		// Make Filename URL
		sprintf(szFilenameURL, "\"file:%s%s\"", F->DirName, F->FileName);

		// Make Folder URL
		sprintf(szFolderURL, "\"file:%s\"", F->DirName);

		// Make Directory Text
		sprintf(szDirText, "%s", F->DirName);

		if(bReturnSmbLinks)
		{
			sprintf(szFilenameURL, "\"smb:%s%s\"", F->DirName, F->FileName);
			sprintf(szFolderURL, "\"smb:%s\"", F->DirName, F->FileName);

			// Now switch all backslash to forward slash
			p = szFilenameURL;
			while(*p != 0)
			{
				if(*p == '\\') *p = '/';
				p++;
			}
			p = szFolderURL;
			while(*p != 0)
			{
				if(*p == '\\') *p = '/';
				p++;
			}
		}
	}

	// Make Filesize Text
	size = (float)F->FileSize / (1024.0f * 1024.0f);
	if(size > 0.5f) 
	{
		size = (float)F->FileSize / (1024.0f * 1024.0f);
		sprintf(szFileSize, "%.2f MB", size);
	}
	else 
	{
		size = (float)F->FileSize / (1024.0f);
		sprintf(szFileSize, "%.2f KB", size);
	}

	printf(szResultLeadin);
	sprintf(szFilenameAndLink, "<A HREF=%s>%s</A>", szFilenameURL, F->FileName);
	printf(szResultFilename, j, szFilenameAndLink);		
	if(bProvideDirLinks || psAllMachines[F->MachineID].m_bIsFTP)
	{
		sprintf(szDirnameAndLink, "<A HREF=%s>%s</A>", szFolderURL, szDirText);
	}
	else
	{
		sprintf(szDirnameAndLink, "%s", szDirText);
	}
	printf(szResultFoldername, szDirnameAndLink);

	printf(szResultSize, szFileSize);
	printf(szResultLeadout);
	return;	
}

bool SS3::CheckBan()
{
  char host_addr[1024];

  if(getenv("REMOTE_ADDR") == NULL)
  {
    return false;
  }

  strncpy(host_addr, getenv("REMOTE_ADDR"), 15);
	host_addr[15] = 0;

  int i=0;

  while(banned[i].type != (-1))
  {
    if(strcmp(host_addr, banned[i].address) == 0)
    {
      switch(banned[i].type)
      {
        case 0:
          return true;
        case 1:
          return true;
      }
    }
    i++;
  }
  return false;
}

void SS3::HTML_Show_Nextlink(char *orig, unsigned int d, unsigned int e, unsigned int f, unsigned int g, unsigned int h)
{
  // Trim original link of "&d" comments
  int i, len;

  len = strlen(orig);
  for(i=0; i<len; i++)
  {
    if(orig[i] == '&' && (orig[i+1] == 'E' || orig[i+1] == 'e'))
    {
      orig[i] = '\0';
    }
  }

	PrintUntilToken("<!--searchcontinue");
	Tchar szContinueLink[1024];
	sprintf(szContinueLink, "/cgi-bin/search.exe?%s&e=%ld&f=%ld&g=%ld&h=%ld", orig, e, f ,g, h);
	ReadLine();
	printf(GetCurrentLine(), szContinueLink);
	ReadLine(); // skip over </searchcontinue-->
	ReadLine(); // skip over three lines of "Total Results"
	ReadLine();
	ReadLine();
}


void SS3::RemoveIllegal(char **List,int *Sizes,int *NumTerms)
{
	int		x,y;
	int		Delta=0;
	char	*Temp;
	int		SizesTemp;
	for(x=0;x<(*NumTerms);x++)
	{
		if(*(List[x])=='-' && *(List[x]+1)=='\"' && *(List[x]+2)=='\"')
		{
			List[x]=NULL;
			Sizes[x]=0;
			Delta++;
		}
		else if(*(List[x])=='\"' && *(List[x]+1)=='\"')
		{
			List[x]=NULL;
			Sizes[x]=0;
			Delta++;
		}
	}
	for(x=0;x<(*NumTerms);x++)
	{
		for(y=0;y<(*NumTerms);y++)
		{
			if(List[x]!=NULL)
			{
				Temp=List[y];
				SizesTemp=Sizes[y];
				List[y]=List[x];
				Sizes[y]=Sizes[x];
				List[x]=Temp;
				Sizes[x]=SizesTemp;
			}
		}
	}
(*NumTerms)-=Delta;
}

static int EvaluateException(unsigned long exceptnum)
{	
	if ( exceptnum == STATUS_INTEGER_OVERFLOW ||
		 exceptnum == STATUS_FLOAT_UNDERFLOW ||
		 exceptnum == STATUS_FLOAT_OVERFLOW )
		 return(EXCEPTION_CONTINUE_SEARCH);

	if (1)
	{ // exception handler is active, try to shut everything down and let the error get caught
		return(EXCEPTION_EXECUTE_HANDLER);
	}
	else
	{ // no exception handler, let it slide through (if the debugger is active, it should catch it)
		return(EXCEPTION_CONTINUE_SEARCH);
	}
}

void vSearch(SS3 *s3, char *arg, char *passed_arg, char *orig_arg)
{
  unsigned long exceptVal;
	__try
	{
	  s3->Search(arg);
	  s3->OutputHTML(passed_arg, orig_arg);
	}
	__except(EvaluateException(exceptVal = GetExceptionCode()))
	{
		switch(exceptVal)
		{
			case EXCEPTION_ACCESS_VIOLATION:				SafeCgiPrintError("Access Violation - error has been logged\n", orig_arg); break;
			case EXCEPTION_ARRAY_BOUNDS_EXCEEDED:		SafeCgiPrintError("Bounds Exception - error has been logged\n", orig_arg); break;
			case EXCEPTION_BREAKPOINT:							SafeCgiPrintError("Breakpoint - error has been logged\n", orig_arg); break;
			default: SafeCgiPrintError("Generic error - error has been logged", orig_arg);
		}
	}
}

int main(int argnum, char **args)
{
	char    arg[2048];
	char    passed_arg[2048];	//copy of arg
	char	orig_arg[2048];
	SS3     CSearch3;

	if(CSearch3.CheckBan() == true)
	{
		return(0);
	}

	memset(CSearch3.F,0,sizeof(CSearch3.F));
#if CMDLINE_TEST
	strcpy(arg, "d=3&a=exe&s=0");
#else
	char    browser[2048];

	if(getenv("HTTP_USER_AGENT") != NULL)
	{
		strncpy(browser, getenv("HTTP_USER_AGENT"), 2048);
		browser[2045] = 0;
		if(strstr(browser, "MSIE") != NULL)
		{
			CSearch3.bProvideDirLinks = true;
		}
		if(strstr(browser, "Konqueror") != NULL)
		{
			CSearch3.bReturnSmbLinks = true;
		}
	}

	if(getenv("QUERY_STRING") == NULL)
	{
		strcpy(arg, "a=errornosearchspecified");
	}
	else
		strncpy(arg, getenv("QUERY_STRING"), 2046);
#endif
	arg[2045] = 0;

	strncpy(orig_arg,arg,2048);
	strupr(arg);
	strncpy(passed_arg,arg,2048);
	
	// -----------------------------------------------------------------------
	// Send info to other window
	TSearchInfo *psSearchInfo;

	psSearchInfo = CSearch3.psSearchInfoBase;
	for(int i=0; i<212; i++)
	{
		if(psSearchInfo->m_bValid == 0)
		{
			if(getenv("REMOTE_ADDR") != NULL)
			{
				strncpy(psSearchInfo->m_szIpAddress, getenv("REMOTE_ADDR"), 15);
				psSearchInfo->m_szIpAddress[15] = 0;
			}
			else
			{
				strcpy(psSearchInfo->m_szIpAddress, "0.0.0.0");
			}

			strcpy(psSearchInfo->m_szSearchString, orig_arg);			
			psSearchInfo->m_bValid = 1;
			break;
		}
	}
	// -----------------------------------------------------------------------

  vSearch(&CSearch3, arg, passed_arg, orig_arg);

	return(1);
}
