/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#ifndef __SEARCH35_H__
#define __SEARCH35_H__

#include "../SsBaCommon.h"
#include "../SsBaMemMap.h"
#include "rand.h"

#define MAXHITS  50		//maximum number of hits to return per search
#define MAXTERMS 32		//maximum number of search terms

#define MISS		0		// No matches
#define HIT			1		// Success
#define ANTIHIT		2		// (-) Hit on negating term
#define ANTIMISS	4		// (-)Miss on negating Term

struct pSrch	
{
	int	DB;				//which Database are we searching
	int MachineIndex;	//index into Machine Array     (index because it's fixed size struct)
	int MinSizeLevel;		// what level of size should we return
	int pStart;			//pointer into Name Array		(ptr because variable length names)
	int pDir;			//pointer into Directory Array
};

struct BanInfo
{
  char address[16];
  int  type;
};


struct FoundFile
{
	char FileName[280];		//hopefully oversize won't kill
	char DirName[280];		//fully quallified directory name
	int  FileSize;			//size of file
	int  MachineID;			//Machine it was found on.
};

class SS3
{

public:

	FoundFile F[MAXHITS];	//Hits placed here
	Rand TheRand;
	bool bProvideDirLinks;
	bool bReturnSmbLinks;
	int	 InstanceID;
	TSearchInfo	*psSearchInfoBase;

	SS3();
	~SS3();

	bool	OpenMachineInfo();            //opens machine info file, mallocs to AllMachines, reads in
	bool	CloseMachineInfo();           //writes back NumMachines, AllMAchines, frees() and fclose()
	bool	OpenDataFiles();			  //open/close databases	
	bool	CloseDataFiles();
	int     Search(char *arg);								   //Search return number of hits
	void    OutputHTML(char *passed_arg, char *arg);				  //print Results
	bool    CheckBan();						//returns true if calling IP is banned
		
private:

	HANDLE			hSearchMonitorMap;
	HANDLE		hMachineMap;				//Memory Map	(machine
	HANDLE		hHandles[2*NUMDATABASES];	//Handles to Memory maps (filenames are[x+0]) (dirnames are x+1)
	TMachine *psAllMachines;
	int			nNumMachines;
	int			*RandomRef;		//Random order of machines

	MemoryMap HtmlMap;
	char *Html;
	int  HtmlSize;
	char *Dir[NUMDATABASES];	//Ptrs to memory maped Directory files put here
	char *Name[NUMDATABASES];	//Ptrs to memory maped Filename data
	unsigned int  FileSize[NUMDATABASES];//FileSize of each Filename data file

	unsigned long   SEED;			//Seed for the PRNG
	int				FoundBaseCount;	//numbers returned results (for next button etc)
	int				NumFound;		//Number of Hits
	pSrch			S;				//Search Pointer

	void    RemoveIllegal(char **List,int *Sizes,int *NumTerms);
	void    Prioritize(char **List, int *Sizes, int NumTerms); //Prioritizes searches
	void    SetSearchOrder();	//sets the order in which machines are searched
	void    LowAI(char **List, int *Sizes, int *NumTerms);	   //A little bit of intelligence
	bool    PreSearch(char *arg,int *NumTerms, char **List);	//Set up a search from string return success
	void    ChangeSpecial(char *String);	//changes %22 into " etc
	bool    DetectQuote(char *String);
	int     NewSearch3(char **String, int *Sizes, int NumTerms, int MinSize); //Actual searching return num hits

	bool	SkipToNextDirectory(int Hint); //moves pStart (FileName POinter) to next directory of files ..Hint should be 0 if no hint
	bool	GetNextDir();	 //gets the next directory pointer
	bool    GetNextMachine(); //finds next machine to search
	int		NameMatch(char *Text, char **String, int *Sizes, int NumTerms, int *pEnd, unsigned long *Mask);
	int		DirMatch(char *Text, char **String, int *Sizes, int NumTerms, int *pEnd, unsigned long *Mask);

	void    HTML_PrintHeader(char *searchterm, int database);
	void    HTML_PrintFooter();
	void    HTML_PrintResult(int j, FoundFile *F);
	void    HTML_Show_Nextlink(char *orig, unsigned int d, unsigned int e, unsigned int f, unsigned int g, unsigned int h);

	void	Prefix(char *Pi, char *P1);

};

#endif