/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "SsBaMemMap.h"

MemoryMap::MemoryMap()
{
	m_MemoryBase = NULL;
	m_FileHandle = 0;
	m_MapHandle = 0;
}
MemoryMap::~MemoryMap()
{
	Release();
}

Tboolean MemoryMap::OpenMap(Tchar *MapName, Tboolean ReadOnly)
{
	Tuint Access = ReadOnly ? FILE_MAP_READ : FILE_MAP_READ | FILE_MAP_WRITE;

	m_MapHandle = OpenFileMapping(
		Access,
		TRUE,
		MapName);

	if(m_MapHandle == 0 || m_MapHandle == INVALID_HANDLE_VALUE)
	{
		m_MapHandle = 0;
		return 0;
	}
	
	m_MemoryBase = (void*)MapViewOfFile(m_MapHandle, Access, 0, 0, 0);
	if(!m_MemoryBase)
	{
		CloseHandle(m_MapHandle);
		m_MapHandle = 0;
		return 0;
	}
	return 1;
}
Tboolean MemoryMap::InitFromFile(Tchar *Filename, Tchar *MapName)
{
	SECURITY_ATTRIBUTES SA;
	SA.bInheritHandle=TRUE;
	SA.lpSecurityDescriptor=NULL;
	SA.nLength=sizeof(SECURITY_ATTRIBUTES);

	m_FileHandle = CreateFile(
															Filename, 
															GENERIC_READ|GENERIC_WRITE,
															FILE_SHARE_READ|FILE_SHARE_WRITE, 
															&SA,
															OPEN_EXISTING, 
															NULL, 
															NULL);	

	if(m_FileHandle == 0 || m_FileHandle == INVALID_HANDLE_VALUE)
	{
		m_FileHandle = 0;
		return 0;
	}

	m_MapHandle = CreateFileMapping(
														m_FileHandle,
														&SA,
														PAGE_READWRITE,
														0,
														0,
														MapName);
	if(m_MapHandle == 0 || m_MapHandle == INVALID_HANDLE_VALUE)
	{
		CloseHandle(m_FileHandle);
		m_FileHandle = 0;
		m_MapHandle = 0;
		return 0;
	}
	
	m_MemoryBase = (void*)MapViewOfFile(m_MapHandle, FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
	if(!m_MemoryBase)
	{
		CloseHandle(m_MapHandle);
		CloseHandle(m_FileHandle);
		m_MapHandle = m_FileHandle = 0;
		return 0;
	}
	return 1;
}
void MemoryMap::FlushFile()
{
	if(m_MemoryBase)
	{
		FlushViewOfFile(m_MemoryBase, 0);
	}
}
void MemoryMap::Release()
{
	if(m_MemoryBase)
	{
		UnmapViewOfFile(m_MemoryBase);
	}
	if(m_FileHandle)
	{
		CloseHandle(m_FileHandle);
	}
	if(m_MapHandle)
	{
		CloseHandle(m_MapHandle);
	}
	m_MemoryBase = NULL;
	m_FileHandle = 0;
	m_MapHandle = 0;
}
void *MemoryMap::GetPointer()
{
	return m_MemoryBase;
}
Tuint MemoryMap::GetFileSize()
{
	return ::GetFileSize(m_FileHandle, 0);
}