/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#ifndef _INC_SS_BA_TYPES
#define _INC_SS_BA_TYPES

// ---------------------------------------------------
//		Type Definitions
// ---------------------------------------------------
#ifndef False
#define False 0
#endif

#ifndef True
#define True 1
#endif

#define	Tuint32		unsigned int
#define Tsint32		signed int
#define Tint32		int

#define Tuint16		unsigned short
#define Tsint16		signed short
#define Tint16		short

#define Tfloat		float

#define Tint			int
#define Tuint			unsigned int
#define Tsint			signed int

#define Tbyte			unsigned char
#define Tword			unsigned short
#define Tdword		unsigned int

#define Tmask			unsigned int
#define Tboolean	unsigned char
#define Tchar			char

// Machines which actually have data associated with them
struct TMachine
{
	Tchar 		m_szIpAddress[32];						// IP Address of this machine
	Tchar 		m_szName[64];									// Machine name
	Tchar 		m_szBaseFtpURL[128];					// Base FTP URL to use 
	Tboolean	m_bIsFTP;											// Is this machine an FTP site?
	Tboolean	m_bMachineUp;									// Is this machine currently on?
	Tfloat		m_fFileBytes;									// How many bytes of files does this computer share?
	Tuint32		m_nNumFiles;									// How many files does this computer share?
	Tuint32		m_mScanFlags;									// Scan Flags for this item
	Tuint32  	m_nNameOffset[NUMDATABASES];	// Offset in Name datafile
	Tuint32	 	m_nDirOffset[NUMDATABASES];		// Offset in Dir datafile
	Tfloat		m_fDatabaseFileBytes[NUMDATABASES];
	Tuint32		m_nDatabaseNumFiles[NUMDATABASES];
	Tuint32		m_nValidMachineIndex;					// Index in Huge Machine Map
};

// These are all the machines known to StrangeSearch
struct TMapMachine
{ 
	Tchar 		m_szIpAddress[17];				// IP Address
	Tchar 		m_szDomain[64];						// Domain name
	Tchar 		m_szNetName[17];					// Netbios Name of the machine
	Tboolean 	m_bMachineUp;							// Is this machine currently on?
	Tuint32  	m_nTimeOfLastScan;				// Time of last scan
	Tuint32	 	m_nTimeOfLastNamePing;		// Time of last ping
  Tuint32		m_nScanFlags;							// Scan Flags for this item
	Tuint32 	m_nValidMachineIndex;			// Index in Huge Machine Map
};

// Search information structure (passed between applications)
struct TSearchInfo
{
	Tchar			m_bValid;										// Is this block valid in memory?
	Tchar			m_szIpAddress[17];					// Searcher's IP Address
	Tchar			m_szSearchString[512];			// Search string
};

enum
{ 
	c_nScanFlagNone				= 0x0000,
	c_nScanFlagIndex			= 0x0001,
	c_nScanFlagIndexFTP		= 0x0002
};

struct TStrangeControl
{
  Tboolean      bSearchingAllowed;
	unsigned int	LastMaintTime;
	Tchar					szRootPath[512];
};
			
#endif
