/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "SsStControl.h"
#include "interface/SsUiApp.h"
#include <direct.h> // for _getcwd()

#define CRITICAL_ERROR(yy) StrangeCriticalError(yy);

SsStControl SsStControl::m_oStControl;

SsStControl *gs_poSsControl = SsStControl::poGetStControl();

// --------- Singleton Interface
SsStControl *SsStControl::poGetStControl()
{
	return &m_oStControl;
}

// --------- Class Implementation
void SsStControl::vInitServer()
{
	Tboolean SearchReady=True;

	if(!m_bIsServerInitialized)
	{
		m_bIsServerInitialized = True;

		g_aszRootPath[0] = 0;

		vLoadConfig();
	
		// Open Datafiles and map them into memory
		if(vOpenFileMapping(c_nSsStControlAll))
		{
			m_poStrangeControl->bSearchingAllowed = 1;
		}
		else
		{
			m_poStrangeControl->bSearchingAllowed = 0;
		}

		strncpy(m_poStrangeControl->szRootPath, g_aszRootPath, 256);

		vStartApp(c_nSsStControlAppSearchMonitor);
	}
}
void SsStControl::vResetValues()
{
	m_bIsServerInitialized = False;

	m_poHugeMachines = NULL;
	m_poValidMachines = NULL;
	m_poSearchMachines = NULL;
	m_nNumMachines = 0;
	m_nNumValidMachines = 0;
	m_nNumSearchMachines = 0;

	for(Tuint i=0; i<24; i++)
	{
		m_pDatabases[i]	= NULL;
	}
}
void SsStControl::vRelease()
{
	if(m_bIsServerInitialized)
	{
		m_bIsServerInitialized = False;
    m_poStrangeControl->bSearchingAllowed = 0;
		vStopApp(c_nSsStControlAppSearchMonitor);
		Sleep(200);
		vStopApp(c_nSsStControlAppSearchMonitor);
		vCloseFileMapping(c_nSsStControlAll);
	}

	vResetValues();
}

void SsStControl::vStartApp(Tuint32 nAppID)
{
	switch(nAppID)
	{
		case c_nSsStControlAppPinger:
			m_oApPinger.vInit();
			m_oApPinger.vBeginExecution();
			m_oApPinger.vAttachToConsoleWindow(SS_VIEW()->hGetConsole(c_nSsUiConsoleScan_Pinger));
			break;
		case c_nSsStControlAppIndexer:
			m_oApIndexer.vInit();
			m_oApIndexer.vBeginExecution();
			m_oApIndexer.vAttachToConsoleWindow(SS_VIEW()->hGetConsole(c_nSsUiConsoleScan_Indexer));
			break;
		case c_nSsStControlAppIpMapperMaint:
			m_oApIpMapperMaint.vInit();
			m_oApIpMapperMaint.vBeginExecution();
			m_oApIpMapperMaint.vAttachToConsoleWindow(SS_VIEW()->hGetConsole(c_nSsUiConsoleScan_IpMapperMaint));
			break;
		case c_nSsStControlAppMaintenance:
			m_oApMaint.vInit();
			m_oApMaint.vBeginExecution();
			m_oApMaint.vAttachToConsoleWindow(SS_VIEW()->hGetConsole(c_nSsUiConsoleSystem));
			break;
		case c_nSsStControlAppSearchMonitor:
			m_oApSearchMonitor.vInit();
			m_oApSearchMonitor.vBeginExecution();
			m_oApSearchMonitor.vAttachToConsoleWindow(SS_VIEW()->hGetConsole(c_nSsUiConsoleSearchMonitor));
			break;
	}
}

void SsStControl::vStopApp(Tuint32 nAppID)
{
	switch(nAppID)
	{
		case c_nSsStControlAppPinger:
			m_oApPinger.vRequestStop();
			break;
		case c_nSsStControlAppIndexer:
			m_oApIndexer.vRequestStop();
			break;
		case c_nSsStControlAppIpMapperMaint:
			m_oApIpMapperMaint.vRequestStop();
			break;
		case c_nSsStControlAppSearchMonitor:
			m_oApSearchMonitor.vRequestStop();
			break;

	}
}

Tboolean SsStControl::bAppRunning(Tuint32 nAppID)
{
	switch(nAppID)
	{
    case c_nSsStControlAppIndexer:				return m_oApIndexer.bIsRunning();
		case c_nSsStControlAppPinger:					return m_oApPinger.bIsRunning();
    case c_nSsStControlAppIpMapperMaint:	return m_oApIpMapperMaint.bIsRunning();
		default:															return False;
	}
}

void SsStControl::vFlushAllMaps()
{
	m_ValidMachineMap.FlushFile();
	m_SearchMachineMap.FlushFile();
}

Tboolean SsStControl::vOpenFileMapping(Tmask mMapAccess)
{
	Tboolean Success=true;
	Tuint i;

	Tchar szFilename[256];

	if(mMapAccess & c_nSsStControlControlMap)
	{
		TStrangeControl sControl;
		memset(&sControl, 0, sizeof(sControl));
		
		sprintf(szFilename, "%s\\data\\Control.dat", g_aszRootPath);
		FILE *pfControlFile = fopen(szFilename, "r");
		if(!pfControlFile)
		{
      pfControlFile = fopen(szFilename, "wb");
			if(pfControlFile == NULL)
			{
				if(!pfControlFile)
				{
					CRITICAL_ERROR("Couldn't generate control.dat -- check paths.\n");
				}			
			}
			fwrite((void *)&sControl, sizeof(sControl), 1, pfControlFile);			
		}
    fclose(pfControlFile);
		
		m_ControlMap.InitFromFile(szFilename, MAPNAME_STRANGECONTROLMAP);
		m_poStrangeControl = (TStrangeControl*)m_ControlMap.GetPointer();
	}
		
	if(mMapAccess & c_nSsStControlSearchMonitorMap)
	{
    TSearchInfo sBlankSearchInfo;
    memset(&sBlankSearchInfo, 0, sizeof(sBlankSearchInfo));
		
		sprintf(szFilename, "%s\\data\\SearchMonitor.dat", g_aszRootPath);
		FILE *pfSearchMonitorFile = fopen(szFilename, "wb");
		if(!pfSearchMonitorFile)
		{
			CRITICAL_ERROR("Couldn't generate SearchMonitor.dat -- check paths.\n");
		}
		for(i=0; i<256; i++)
		{
			fwrite((void *)&sBlankSearchInfo, sizeof(TSearchInfo), 1, pfSearchMonitorFile);
		}
		fclose(pfSearchMonitorFile);
		
		m_SearchMonitorMap.InitFromFile(szFilename, MAPNAME_SEARCHMONITORMAP);
		m_poSearchInfo = (TSearchInfo*)m_SearchMonitorMap.GetPointer();
	}
		
	if(mMapAccess & c_nSsStControlValidMachineMap)
	{
		sprintf(szFilename, "%s\\data\\ValidMachineMap.dat", g_aszRootPath);	
		if(!m_ValidMachineMap.InitFromFile(szFilename, MAPNAME_VALIDMACHINEMAP))
		{
			// ValidMachine file missing.
			CRITICAL_ERROR("ValidMachineMap not found (or empty). Please generate before running StrangeSearch.");
		}
		else
		{
			m_poValidMachines = (TMapMachine *)m_ValidMachineMap.GetPointer();
			m_nNumValidMachines = m_ValidMachineMap.GetFileSize() / sizeof(TMapMachine);
		}
	}
		
	if(mMapAccess & c_nSsStControlSearchMachineMap)	
	{
		sprintf(szFilename, "%s\\data\\SearchMachineDB.dat", g_aszRootPath);	
		if(!m_SearchMachineMap.InitFromFile(szFilename, MAPNAME_SEARCHMACHINEMAP))
		{
			m_poSearchMachines = NULL;
			m_nNumSearchMachines = 0;
			Success = False;
			
		}
		else
		{
			m_poSearchMachines = (TMachine *)m_SearchMachineMap.GetPointer();
			m_nNumSearchMachines = m_SearchMachineMap.GetFileSize() / sizeof(TMachine);
		}
	}

	if(mMapAccess & c_nSsStControlSearchHtml)	
	{
		sprintf(szFilename, "%s\\data\\SearchTemplate.html", g_aszRootPath);	
		if(!m_SearchHtml.InitFromFile(szFilename, MAPNAME_STRANGEHTML))
		{
			CRITICAL_ERROR("StrangeSearch requires SearchTemplate.html to be present in data directory.");
			m_pszSearchHtml = NULL;
			Success = False;			
		}
		else
		{
			m_pszSearchHtml = (Tchar *)m_SearchHtml.GetPointer();
		}
	}

	if(mMapAccess & c_nSsStControlDatabases)
	{
		for(i=0; i<NUMDATABASES*2; i++)
		{
			sprintf(szFilename, "%s\\data\\search\\%s", g_aszRootPath, g_aszFilenames[i]);
			
			if(!m_DatabaseMaps[i].InitFromFile(szFilename, g_aszMapNames[i]))
			{
				m_pDatabases[i] = NULL;
			}
			else
			{
				m_pDatabases[i] = (Tchar *)m_DatabaseMaps[i].GetPointer();
			}
		}
	}
	return Success;
	
}

void SsStControl::vCloseFileMapping(Tmask mMapAccess)
{
	if((mMapAccess & c_nSsStControlValidMachineMap) && m_ValidMachineMap.GetPointer())
	{
		m_ValidMachineMap.Release();
		m_poValidMachines = NULL;
	}
	if((mMapAccess & c_nSsStControlSearchMachineMap) && m_SearchMachineMap.GetPointer())
	{
		m_SearchMachineMap.Release();
		m_poSearchMachines = NULL;
	}
	if(mMapAccess & c_nSsStControlDatabases)
	{
		for(Tuint i=0; i<24; i++)
		{
			if(m_DatabaseMaps[i].GetPointer() != NULL)
			{
				m_DatabaseMaps[i].Release();
				m_pDatabases[i] = NULL;
			}
		}
	}
	if((mMapAccess & c_nSsStControlControlMap) && m_ControlMap.GetPointer())
	{
		m_ControlMap.FlushFile();
		m_ControlMap.Release();
	}
	if((mMapAccess & c_nSsStControlSearchMonitorMap) && m_SearchMonitorMap.GetPointer())
	{
		m_SearchMonitorMap.FlushFile();
		m_SearchMonitorMap.Release();
	}
}

void SsStControl::vLoadConfig()
{
	Tchar Variable[256];
	Tchar Value[256];

	FILE *f = fopen("strange.ini", "r");

	if(!f)
	{
		CRITICAL_ERROR("strange.ini could not be found");
	}		

	g_aszRootPath[0] = 0;
	g_nIndexRefreshSeconds = 0;
	g_nIndexRefreshSeconds = 0;
	g_nNameExpireSeconds = 0;

	while(1)
	{
		ReadConfigLine(f, Variable, Value);
		if(Variable[0] == 0)
			break;

		if(stricmp(Variable, "IndexRefresh") == 0)
		{
			g_nIndexRefreshSeconds = atol(Value);
		}
		if(stricmp(Variable, "IndexExpire") == 0)
		{
			g_nIndexExpireSeconds = atol(Value);
		}
		if(stricmp(Variable, "NameExpire") == 0)
		{
			g_nNameExpireSeconds = atol(Value);
		}
	}
	fclose(f);

	// Read Time Delay Values

	if(g_nIndexRefreshSeconds == 0 || 
		g_nIndexExpireSeconds == 0 ||
		g_nNameExpireSeconds == 0)
	{
		CRITICAL_ERROR("Invalid (or missing) time values in strange.ini");
	}
	_getcwd(g_aszRootPath, 258);
}

void SsStControl::ReadConfigLine(FILE *iniFile, char *Variable, char *Value)
{
	Tchar buffer[258];
	memset(buffer, 0, sizeof(buffer));
	fgets(buffer, 256, iniFile);
	if(feof(iniFile))
	{
		Variable[0] = 0;
		Value[0] = 0;
		return;
	}
	strcat(buffer, "\n");
	if(buffer[0] == '[')
	{
		ReadConfigLine(iniFile, Variable, Value);
		return;
	}
	Tchar *First, *Second;

	First = buffer;
	strtok(buffer, "=\n");
	Second = strtok(NULL, "\n");

	// Trim leading
	if(First) while(*First == ' ') First++;
	if(Second) while(*Second == ' ') Second++;

	if(First)
		strncpy(Variable, First, 256);
	else
		Variable[0] = 0;

	if(Second)
		strncpy(Value, Second, 256);
	else
		Value[0] = 0;
}