/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#ifndef _INC_SS_ST_CONTROL
#define _INC_SS_ST_CONTROL

#include "SsBaCommon.h"
#include "apps/scan/SsApScPinger.h"
#include "apps/scan/SsApScIpMapMaint.h"
#include "apps/SsApIndexer.h"
#include "apps/SsApMaint.h"
#include "apps/SsApSearchMonitor.h"
#include "SsBaMemMap.h"

enum
{
	c_nSsStControlNone									= 0x0000,
	c_nSsStControlHugeMachineMap				= 0x0001,
	c_nSsStControlValidMachineMap				= 0x0002,
	c_nSsStControlSearchMachineMap			= 0x0004,
	c_nSsStControlDatabases							= 0x0008,
	c_nSsStControlControlMap						= 0x0010,
	c_nSsStControlSearchMonitorMap			= 0x0020,
	c_nSsStControlSearchHtml            = 0x0040,
	c_nSsStControlSearchError           = 0x0080,
	c_nSsStControlAll										= 0xFFFF
};

enum
{
	c_nSsStControlAppPinger							= 0,
	c_nSsStControlAppPingerMaint				= 1,
	c_nSsStControlAppIndexer						= 2,
	c_nSsStControlAppIpMapperMaint			= 4,
	c_nSsStControlAppMaintenance				= 5,
	c_nSsStControlAppStat								= 6,
	c_nSsStControlAppSearchMonitor			= 7
};

class SsStControl;
extern SsStControl *gs_poSsControl;

class SsStControl
{
protected:
  // ------------------------------------------ Singleton Protected Variables
	SsStControl()							
	{ 
		poGetStControl()->vResetValues(); 
		gs_poSsControl = this;			
	}
	static SsStControl m_oStControl;
	// ------------------------------------------

	Tboolean 	m_bIsServerInitialized;					// Has this class been initialized?

	TMapMachine *m_poHugeMachines;			// Pointer to the Huge Machine Map in memory
	TMapMachine *m_poValidMachines;			// Pointer to the Valid Machine Map in memory
	TMachine		*m_poSearchMachines;		// Pointer to the Search Machine DB Map in memory
	Tuint				m_nNumMachines;					// # of Machines in Huge Machine Map
	Tuint				m_nNumValidMachines;		// # of Machines in Valid Machine Map
	Tuint				m_nNumSearchMachines;		// # of Machines in the Search DB Map

	MemoryMap		m_ValidMachineMap;
	MemoryMap		m_SearchMachineMap;
	MemoryMap		m_ControlMap;
	MemoryMap		m_SearchMonitorMap;
	MemoryMap		m_SearchHtml, m_SearchErrorHtml;
	MemoryMap		m_DatabaseMaps[24];

	Tchar				*m_pDatabases[24];
	Tchar				*m_pszSearchHtml, *m_pszSearchErrorHtml;
	TSearchInfo	*m_poSearchInfo;
  TStrangeControl *m_poStrangeControl;

	SsApScPinger 				m_oApPinger;
	SsApIndexer				  m_oApIndexer;
	SsApScIpMapMaint 		m_oApIpMapperMaint;
	SsApMaint						m_oApMaint;
	SsApSearchMonitor		m_oApSearchMonitor;

public:
	static SsStControl *poGetStControl();
	virtual ~SsStControl()			{	poGetStControl()->vRelease(); }

	void vInitServer();
	void vResetValues();
	void vRelease();

	void vLoadConfig();
	void ReadConfigLine(FILE *iniFile, char *Variable, char *Value);

	void vStartApp(Tuint32 nAppID);
	void vStopApp(Tuint32 nAppID);
	Tboolean bAppRunning(Tuint32 nAppID);

	void			vFlushAllMaps();
	Tboolean	vOpenFileMapping(Tmask mMapAccess);
	void			vCloseFileMapping(Tmask mMapAccess);

	TMapMachine *poGetValidMachine(Tuint nIndex)
	{
		if(!m_poValidMachines || (nIndex >= m_nNumValidMachines))
		{
			return NULL;
		}
		return &m_poValidMachines[nIndex];
	}
	TMachine *poGetSearchMachine(Tuint nIndex)
	{
		if(!m_poSearchMachines || (nIndex >= m_nNumSearchMachines))
		{
			return NULL;
		}
		return &m_poSearchMachines[nIndex];
	}
	TSearchInfo *poGetSearchInfo(Tuint nIndex)
	{
		if(!m_poSearchInfo || (nIndex >= 256))
		{
			return NULL;
		}
		return &m_poSearchInfo[nIndex];
	}
  TStrangeControl *poGetStrangeControl()
  {
    return m_poStrangeControl;
  }

	inline Tboolean bIsInitialized()				{ return m_bIsServerInitialized; }
	inline Tuint		nGetNumMachines()				{ return m_nNumMachines; }
	inline Tuint		nGetNumValidMachines()	{ return m_nNumValidMachines; }
  inline Tuint		nGetNumSearchMachines()	{ return m_nNumSearchMachines; }
};

#define SS_CONTROL() SsStControl::poGetStControl()
			
#endif
