/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "../SsBaCommon.h"
#include "SsUiWndConsole.h"
#include "richedit.h"

IMPLEMENT_DYNCREATE(SsUiWndConsole, CFrameWnd);

BEGIN_MESSAGE_MAP(SsUiWndConsole, CFrameWnd)
	ON_WM_SIZE()
END_MESSAGE_MAP()

SsUiWndConsole::SsUiWndConsole()
{
	m_nCharCount = 0;
}

SsUiWndConsole::~SsUiWndConsole()
{
}


BOOL SsUiWndConsole::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	// Create the control
	CRect myWindow;
	GetWindowRect(&myWindow);

	m_Display.Create(ECO_READONLY | ES_MULTILINE | ECO_WANTRETURN | ECO_AUTOVSCROLL | WS_VISIBLE, CRect(0,0,myWindow.Width(), myWindow.Height()), this, 10003);

	CHARFORMAT cf;

	memset(&cf, 0, sizeof(cf));
	cf.cbSize = sizeof (CHARFORMAT);  
	cf.dwMask = CFM_FACE | CFM_SIZE; 
	cf.yHeight = 180; 
	sprintf(cf.szFaceName, "Fixedsys"); 
	m_Display.SetBackgroundColor(False, 0);
	m_Display.SetDefaultCharFormat(cf); 
	m_Display.SetSel(0, -1);
	m_Display.ReplaceSel("");
	vAddString("");
	
	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void SsUiWndConsole::OnSize( UINT nType, int cx, int cy )
{
	m_Display.MoveWindow(0,0,cx,cy);
	//MoveWindow(0,0,cx,cy);
}

BOOL SsUiWndConsole::OnWndMsg(UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	switch(message)
	{
		case c_nSsApMsgPrintString:
			vAddString((Tchar *)wParam);
			return TRUE;
		case c_nSsApMsgResetConsole:
			vResetConsole();
			return TRUE;
	}

	return CFrameWnd::OnWndMsg(message, wParam, lParam, pResult);
}

void SsUiWndConsole::vResetConsole()
{
	m_Display.SetSel(0, -1);
	m_Display.ReplaceSel("");
	m_nCharCount = 0;
	vAddString("");
}

void SsUiWndConsole::vAddText(Tchar *szString, COLORREF crTextColor)
{
	m_Display.SetSel(m_nCharCount, m_nCharCount);
	m_Display.ReplaceSel((LPCTSTR)szString);

	int iStartPos = m_nCharCount;
	int iEndPos = m_nCharCount + strlen(szString);

	CHARFORMAT cf;

	cf.cbSize = sizeof(CHARFORMAT);
	cf.dwMask = CFM_COLOR | CFM_BOLD | CFM_UNDERLINE;
	cf.dwEffects = (unsigned long)~(CFE_UNDERLINE | CFE_BOLD | CFE_AUTOCOLOR);
	cf.crTextColor = crTextColor;
	//cf.dwEffects |= bUnderLine ? CFE_UNDERLINE : cf.dwEffects ;
	//cf.dwEffects |= bBold ? CFE_BOLD : cf.dwEffects;

	m_Display.SetSel(iStartPos, iEndPos);
	m_Display.SetSelectionCharFormat(cf);
	m_Display.HideSelection(TRUE, FALSE);

	m_Display.LineScroll(1);	

	m_nCharCount += strlen(szString);
}

void SsUiWndConsole::vAddString(Tchar *szString)
{
	static Tchar szBuffer[1024];
	Tuint nBufferIndex, nStringIndex;
	COLORREF crCurrentColor = 0x00FFFFFF;
	char cColorChar;

	for(nBufferIndex = 0, nStringIndex = 0; nStringIndex < strlen(szString); nStringIndex++)
	{
		if(szString[nStringIndex] == '&')
		{
			if(nBufferIndex > 0)
			{
				vAddText(szBuffer, crCurrentColor);
				nBufferIndex = 0;
				szBuffer[0] = 0;
			}
			// Grab next character
			cColorChar = szString[nStringIndex+1];
			nStringIndex+=2;
			switch(cColorChar)
			{
				case 'b': crCurrentColor = 0x00FF8000; break;
				case 'R': crCurrentColor = 0x000000FF; break;
				case 'B': crCurrentColor = 0x00FF0000; break;
				case 'g': crCurrentColor = 0x00808080; break;
				default:  crCurrentColor = 0x00FFFFFF; break;
			}
			continue;
		}
		else
		{
			szBuffer[nBufferIndex++] = szString[nStringIndex];
			szBuffer[nBufferIndex] = 0;
		}
	}
	if(nBufferIndex > 0)
	{
		vAddText(szBuffer, crCurrentColor);
		nBufferIndex = 0;
		szBuffer[0] = 0;
	}
}
