/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "../SsBaCommon.h"
#include "SsUiWndMachines.h"
#include "SsUiWndView.h"
#include "SsUiApp.h"
#include "SsStControl.h"
#include "netutils/SsNuPinger.h"

#include <commctrl.h>

const Tuint c_nSsUiMachinesID = 10003;


IMPLEMENT_DYNCREATE(SsUiWndMachines, CFrameWnd);

BEGIN_MESSAGE_MAP(SsUiWndMachines, CFrameWnd)
	ON_WM_SIZE()
	ON_NOTIFY(LVN_KEYDOWN, c_nSsUiMachinesID, OnKeyDown)
END_MESSAGE_MAP();

BOOL SsUiWndMachines::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	CRect oClientRect;
	GetWindowRect(&oClientRect);

	LV_COLUMN sColumn;
	char szColumnText[128];

	
	m_oMachines.Create(LVS_REPORT | LVS_NOSORTHEADER | WS_VISIBLE, oClientRect, this, c_nSsUiMachinesID);

	sColumn.mask = LVCF_TEXT | LVCF_WIDTH;
	sColumn.pszText = szColumnText;

	sColumn.cx = 100;
	strcpy(szColumnText, "IP Address");
	m_oMachines.InsertColumn(0, &sColumn);

	strcpy(szColumnText, "Machine Name");
	sColumn.cx = 300;
	m_oMachines.InsertColumn(1, &sColumn);

	strcpy(szColumnText, "Domain Name");
	sColumn.cx = 200;
	m_oMachines.InsertColumn(2, &sColumn);

	strcpy(szColumnText, "Scan Flags");
	sColumn.cx = 75;
	m_oMachines.InsertColumn(3, &sColumn);


	strcpy(szColumnText, "Status");
	sColumn.cx = 75;
	m_oMachines.InsertColumn(4, &sColumn);

	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void SsUiWndMachines::OnSize( UINT nType, int cx, int cy )
{
	m_oMachines.MoveWindow(0,0,cx,cy,true);
}

void SsUiWndMachines::vPreActivate()
{
	TMapMachine *psCurMachine;

	m_oMachines.DeleteAllItems();
	for(Tuint i=0; i<SS_CONTROL()->nGetNumValidMachines(); i++)
	{
		psCurMachine = SS_CONTROL()->poGetValidMachine(i);
		m_oMachines.InsertItem(LVIF_PARAM | LVIF_TEXT, i, psCurMachine->m_szIpAddress, 0, 0, 0, (LPARAM)psCurMachine);
		m_oMachines.SetItem(i, 1, LVIF_TEXT, psCurMachine->m_szNetName, 0, 0, 0, 0);
		m_oMachines.SetItem(i, 2, LVIF_TEXT, psCurMachine->m_szDomain, 0, 0, 0, 0);
		m_oMachines.SetItem(i, 3, LVIF_TEXT, psCurMachine->m_nScanFlags == 1 ? "Normal" : "Exception", 0, 0, 0, 0);
		m_oMachines.SetItem(i, 4, LVIF_TEXT, psCurMachine->m_bMachineUp ? "Online" : "Offline", 0, 0, 0, 0);
	}

}

void SsUiWndMachines::OnKeyDown(NMHDR * pNotifyStruct, LRESULT * result)
{
	NMLVKEYDOWN *psKeyDownNotification = (NMLVKEYDOWN *)pNotifyStruct;
	POSITION psPosition = m_oMachines.GetFirstSelectedItemPosition();
	//int nSelectedItem = m_oMachines.GetFirstSelectedItemPosition();
	LVITEM psItem;
	TMapMachine *psCurMachine;

	psItem.mask = LVIF_PARAM;
	psItem.iItem = (Tuint)psPosition-1;
	
	switch(psKeyDownNotification->wVKey)
	{
		case VK_F5:
			m_oMachines.GetItem(&psItem);
			if(psItem.lParam != NULL)
			{
				SsNuPinger oPingUtil;
				char szComputerName[256];

				psCurMachine = (TMapMachine *)psItem.lParam;
				// Try to retrieve the name of this computer
				if(oPingUtil.bPingByIP(psCurMachine->m_szIpAddress))
				{
					if(oPingUtil.bGetNetbiosName(szComputerName, psCurMachine->m_szIpAddress))
					{
						strcpy(psCurMachine->m_szNetName, szComputerName);
						m_oMachines.SetItem((Tuint)psPosition-1, 1, LVIF_TEXT, psCurMachine->m_szNetName, 0, 0, 0, 0);
					}
				}
			}
			break;
		case VK_F6:
			m_oMachines.GetItem(&psItem);
			if(psItem.lParam != NULL)
			{
				psCurMachine = (TMapMachine *)psItem.lParam;
				strcpy(psCurMachine->m_szNetName, "");
				m_oMachines.SetItem((Tuint)psPosition-1, 1, LVIF_TEXT, psCurMachine->m_szNetName, 0, 0, 0, 0);
			}
			break;

		case VK_F9:
			SS_CONTROL()->vStopApp(c_nSsStControlAppSearchMonitor);
			break;
	}
				

}