/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */

#include "../SsBaCommon.h"
#include "SsStControl.h"
#include "SsUiWndScanControl.h"
#include "SsUiWndView.h"
#include "SsUiApp.h"

IMPLEMENT_DYNCREATE(SsUiWndScanControl, CFrameWnd);

BEGIN_MESSAGE_MAP(SsUiWndScanControl, CFrameWnd)
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_COMMAND_RANGE(20001, 20007, OnCommand)
END_MESSAGE_MAP()

SsUiWndScanControl::SsUiWndScanControl()
{
}

SsUiWndScanControl::~SsUiWndScanControl()
{
}


BOOL SsUiWndScanControl::OnCreateClient( LPCREATESTRUCT lpcs, CCreateContext* pContext )
{
	m_nTimer = SetTimer(1, 1000, 0);

	// Create the control
	CRect oMyWindow;
	GetWindowRect(&oMyWindow);

	m_oFont.CreateFont(-12, 0, 0, 0, 0, FALSE, FALSE,0,0,0,0,0,0, "MS Sans Serif");

	Tuint ButtonTop = 35;
	m_oScreenDescription.Create("StrangeSearch 3.5 System Console", SS_CENTER | WS_VISIBLE, CRect(0, 0, oMyWindow.Width(), 30), this);
	m_oPingButton.Create("Start Ping Service", BS_PUSHBUTTON | WS_VISIBLE, CRect(5,ButtonTop,200,ButtonTop+25), this, 20001); ButtonTop += 30;
	m_oIndexButton.Create("Start Index Service", BS_PUSHBUTTON | WS_VISIBLE, CRect(5,ButtonTop,200,ButtonTop+25), this, 20003); ButtonTop += 30;
	m_oIPMapMaintButton.Create("Start IP Map Maint Service", BS_PUSHBUTTON | WS_VISIBLE, CRect(5,ButtonTop,200,ButtonTop+25), this, 20005); ButtonTop += 30;
	m_oMaintenanceButton.Create("Perform Maintenance", BS_PUSHBUTTON | WS_VISIBLE, CRect(5,ButtonTop,200,ButtonTop+25), this, 20006); ButtonTop += 30;
	m_oSearchingAllowed.Create("Disable Searching", BS_PUSHBUTTON | WS_VISIBLE, CRect(5,ButtonTop,200,ButtonTop+25), this, 20007);

  m_oScreenDescription.SetFont(&m_oFont);

	m_oPingButton.SetFont(&m_oFont);
	m_oIndexButton.SetFont(&m_oFont);
	m_oIPMapMaintButton.SetFont(&m_oFont);
	m_oMaintenanceButton.SetFont(&m_oFont);
	m_oSearchingAllowed.SetFont(&m_oFont);

	m_oScreenDescription.SetTextColor(::GetSysColor(COLOR_INFOTEXT));
	m_oScreenDescription.SetBkColor(::GetSysColor(COLOR_INFOBK));

  CTime oTime = CTime::GetCurrentTime();
  CString oBootTime = oTime.Format("StrangeSearch 3.5 System Console\nOnline at %m.%d.%Y %I:%M%p");
	m_oScreenDescription.SetWindowText(oBootTime);


	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void SsUiWndScanControl::vUpdateButtonText()
{
	if(!SS_CONTROL()->bIsInitialized())
	{
		return;
	}

	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppPinger))
	{
		m_oPingButton.SetWindowText("Stop Ping Service");
	}
	else
	{
		m_oPingButton.SetWindowText("Start Ping Service");
    if(gs_poSsControl->poGetStrangeControl()->bSearchingAllowed)
    {
			// Uncomment to enable auto-startup
      gs_poSsControl->vStartApp(c_nSsStControlAppPinger);
    }
	}

	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIndexer))
	{
		m_oIndexButton.SetWindowText("Stop Indexing Service");
	}
	else
	{
		m_oIndexButton.SetWindowText("Start Indexing Service");
    if(gs_poSsControl->poGetStrangeControl()->bSearchingAllowed)
    {
			// Uncomment to enable auto-startup
      gs_poSsControl->vStartApp(c_nSsStControlAppIndexer);
    }
	}

	if(SS_CONTROL()->bAppRunning(c_nSsStControlAppIpMapperMaint))
	{
		m_oIPMapMaintButton.SetWindowText("Stop IP Mapping Maint Service");
	}
	else
	{
		m_oIPMapMaintButton.SetWindowText("Start IP Mapping Maint Service");
    if(gs_poSsControl->poGetStrangeControl()->bSearchingAllowed)
    {
			// Uncomment to enable auto-startup
      gs_poSsControl->vStartApp(c_nSsStControlAppIpMapperMaint);
    }
	}
	if(gs_poSsControl->poGetStrangeControl()->bSearchingAllowed)
	{
		m_oSearchingAllowed.SetWindowText("Disable Searching");
	}
	else
	{
		m_oSearchingAllowed.SetWindowText("Enable Searching");
	}
}

BOOL SsUiWndScanControl::OnCommand(WPARAM wParam, LPARAM lParam)
{
	switch(wParam)
	{
		case 20001:
			if(!SS_CONTROL()->bAppRunning(c_nSsStControlAppPinger))
			{
				SS_CONTROL()->vStartApp(c_nSsStControlAppPinger);
			}
			else
			{
				SS_CONTROL()->vStopApp(c_nSsStControlAppPinger);
			}		
			break;
		case 20003:
			if(!SS_CONTROL()->bAppRunning(c_nSsStControlAppIndexer))
			{
				SS_CONTROL()->vStartApp(c_nSsStControlAppIndexer);
			}
			else
			{
				SS_CONTROL()->vStopApp(c_nSsStControlAppIndexer);
			}		
			break;
		case 20005:
			if(!SS_CONTROL()->bAppRunning(c_nSsStControlAppIpMapperMaint))
			{
				SS_CONTROL()->vStartApp(c_nSsStControlAppIpMapperMaint);
			}
			else
			{
				SS_CONTROL()->vStopApp(c_nSsStControlAppIpMapperMaint);
			}		
			break;
		case 20006:
			if(!SS_CONTROL()->bAppRunning(c_nSsStControlAppMaintenance))
			{
				SS_CONTROL()->vStartApp(c_nSsStControlAppMaintenance);
			}
			else
			{
				SS_CONTROL()->vStopApp(c_nSsStControlAppMaintenance);
			}		
			SS_VIEW()->vSetActiveView(c_nSsUiConsoleSystem);
			break;
		case 20007:
			if(gs_poSsControl->poGetStrangeControl()->bSearchingAllowed)
			{
				gs_poSsControl->poGetStrangeControl()->bSearchingAllowed = False;
			}
			else
			{
				gs_poSsControl->poGetStrangeControl()->bSearchingAllowed = True;
			}
	}
	return 0;
}

afx_msg void SsUiWndScanControl::OnTimer(UINT nIDEvent)
{
	switch(nIDEvent)
	{
		case 1:
			vUpdateButtonText();
			break;
	}
}
