/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include "../SsBaCommon.h"
#include "SsUiWndView.h"
#include "SsUiWndConsole.h"
#include "SsUiWndScanControl.h"
#include "SsUiWndMachines.h"
#include "SsStControl.h"
#include "SsUiApp.h"

IMPLEMENT_DYNCREATE(SsUiWndView, CFrameWnd);

BEGIN_MESSAGE_MAP(SsUiWndView, CFrameWnd)
	ON_WM_SIZE()
	ON_WM_DESTROY()
END_MESSAGE_MAP()

SsUiWndView::SsUiWndView()
{
	m_nActiveConsoles = 0;
  m_apoConsoles = NULL;
	m_poScanControl = NULL;
}

SsUiWndView::~SsUiWndView()
{
	vRelease();
}

BOOL SsUiWndView::OnCreateClient( LPCREATESTRUCT lpcs, CCreateContext* pContext )
{
	CRect oClientRect;
	GetWindowRect(&oClientRect);

  m_apoConsoles = new SsUiWndConsole[c_nSsUiConsoleScan_Active];
	m_apoConsoles[c_nSsUiConsoleSystem].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
  m_apoConsoles[c_nSsUiConsoleScan_Pinger].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
  m_apoConsoles[c_nSsUiConsoleScan_PingerMaint].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
  m_apoConsoles[c_nSsUiConsoleScan_Indexer].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
  m_apoConsoles[c_nSsUiConsoleScan_IpMapperMaint].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
	m_apoConsoles[c_nSsUiConsoleSearchMonitor].Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);

	m_poScanControl = new SsUiWndScanControl;
  m_poScanControl->Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);

	m_poMachines = new SsUiWndMachines;
	m_poMachines->Create(NULL, "test", WS_CHILD, CRect(0,0,oClientRect.Width(),oClientRect.Height()), this, NULL, 0, NULL);
	
	m_nActiveConsoles = c_nSsUiConsoleScan_Active;

	m_poScanControl->ShowWindow(TRUE);

	return CFrameWnd::OnCreateClient(lpcs, pContext);
}

void SsUiWndView::OnSize( UINT nType, int cx, int cy )
{
	CRect oClientRect;
	GetClientRect(&oClientRect);

	Tint nColWidth, nColMin;
	Tint nRowWidth, nRowMin;

	g_poMainWnd->poGetSplitter()->GetColumnInfo(1, nColWidth, nColMin);
	g_poMainWnd->poGetSplitter()->GetRowInfo(0, nRowWidth, nRowMin);

	MoveWindow(159, 0, nColWidth, nRowWidth);

	Tuint nCurConsole;
	for(nCurConsole = 0; nCurConsole < m_nActiveConsoles; nCurConsole++)
	{
		m_apoConsoles[nCurConsole].MoveWindow(0, 0, nColWidth, nRowWidth);
	}
	m_poScanControl->MoveWindow(0,0, nColWidth, nRowWidth);
	m_poMachines->MoveWindow(0,0, nColWidth, nRowWidth);
}

void SsUiWndView::OnDestroy()
{
	vRelease();
	SS_CONTROL()->vRelease();
}

void SsUiWndView::vSetActiveView(Tuint nConsoleIndex)
{
	Tuint nCurConsole;
	for(nCurConsole = 0; nCurConsole < m_nActiveConsoles; nCurConsole++)
	{
		m_apoConsoles[nCurConsole].ShowWindow(nCurConsole == nConsoleIndex ? True : False);
	}
	m_poScanControl->ShowWindow( (nConsoleIndex == c_nSsUiConsoleScanControl) );
	if(nConsoleIndex == c_nSsUiConsoleMachines)
	{
		m_poMachines->vPreActivate();
		m_poMachines->ShowWindow(1);
	}
	else
	{
		m_poMachines->ShowWindow(0);
	}
}

void SsUiWndView::vRelease()
{
	if(m_nActiveConsoles != 0)
	{
		delete [] m_apoConsoles;
		m_apoConsoles = NULL;
		m_nActiveConsoles = 0;
	}
	if(m_poScanControl)
	{
		m_poScanControl->DestroyWindow();
		m_poScanControl = NULL;
	}
}