/*
 * This file is part of the StrangeSearch project
 * Copyright 2001 Dan Dunham and Jeremy Davis
 *
 * It is distributed under the GNU Public Licence (GPL) version 2.  
 * See http://www.gnu.org/copyleft/gpl.html for further details 
 * of the GPL.
 */
#include <stdio.h>
#include <string.h>
#include "../../SsBaCommon.h"
#include "../../NetUtils/SsNuPinger.h"
#include "../../SsBaMemMap.h"
#include <process.h>

FILE *ValidMap;
Tuint NumMachines=0;
TMapMachine *ValidMemoryMap;
Tuint NumThreadsRunning;
Tuint ThreadsDone[16];
Tchar ValidDomain[64];

SsNuPinger Pinger;

void InitMapMaker()
{
	TMapMachine CurMachine;
	
	fseek(ValidMap, 0, SEEK_SET);;
	fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	while(!feof(ValidMap))
	{
		NumMachines++;
		fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	}
}

void PrintAddresses()
{
	TMapMachine CurMachine;

	fseek(ValidMap, 0, SEEK_SET);;
	fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	while(!feof(ValidMap))
	{
		printf("%s %s %s\n", CurMachine.m_szIpAddress, CurMachine.m_szDomain, CurMachine.m_szNetName);
		fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	}
}

void ScanMachineThread(void *index)
{
	SsNuPinger Pinger;
	Tuint threadNum = (Tuint)index;
	Tuint MachineNum = threadNum;
	//printf("thread %d spawned\n", threadNum);

	while(MachineNum < NumMachines)
	{
		TMapMachine *CurMachine = ValidMemoryMap;
		CurMachine += MachineNum;
		Pinger.bRetrieveName(CurMachine->m_szDomain, CurMachine->m_szIpAddress);
		Pinger.bGetNetbiosName(CurMachine->m_szNetName, CurMachine->m_szIpAddress);
		printf("[%d] %s %s %s\n", threadNum, CurMachine->m_szIpAddress, CurMachine->m_szDomain, CurMachine->m_szNetName);
		CurMachine->m_nScanFlags = 1;

		MachineNum += NumThreadsRunning;
	}
	ThreadsDone[threadNum] = 1;
	//printf("thread %d finished\n", threadNum);
}

void ScanMachines(Tuint ThreadCount)
{
	Tuint Index=0;
	
	MemoryMap ValidMachineFile;

	if(ThreadCount > 16) ThreadCount = 16;

	ValidMachineFile.InitFromFile("ValidMachineMap.dat", "SS35_MAPMAKER_ValidMACHINE");
	ValidMemoryMap = (TMapMachine*)ValidMachineFile.GetPointer();

	if(!ValidMemoryMap)
	{
		printf("Couldn't open ValidMachineMap.dat");
		return;
	}
	NumMachines = ValidMachineFile.GetFileSize() / sizeof(TMapMachine);
	NumThreadsRunning = ThreadCount;
	for(Tuint index=0; index<ThreadCount; index++)
	{
		_beginthread(ScanMachineThread, 0, (void*)index);
	}

	while(1)
	{
		Tuint NumDone = 0;
		Tuint idx=0;
		for(idx=0; idx<16; idx++)
		{
			if(ThreadsDone[idx] == 1)
			{
				NumDone++;
			}
		}
		if(NumDone == NumThreadsRunning)
		{
			break;
		}
		Sleep(500);
	}

	
	ValidMachineFile.Release();
}

Tint FindIpAddress(Tuint ip[4])
{
	Tchar checkAddress[16];
	TMapMachine CurMachine;
	Tuint CurIndex=0;
	sprintf(checkAddress, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);

	fseek(ValidMap, 0, SEEK_SET);;
	fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	while(!feof(ValidMap))
	{
		if(stricmp(checkAddress, CurMachine.m_szIpAddress) == 0)
		{
			return CurIndex;
		}
		CurIndex++;
		fread((void *)&CurMachine, 1, sizeof(CurMachine), ValidMap);
	}

	return -1;
}

void AddIpAddress(Tuint ip[4])
{
	TMapMachine NewMachine;

	if((ip[0] == 0 && ip[1] == 0 && ip[2] == 0 &&  ip[3] == 0) || (ip[0] == 0))
	{
		printf("didn't add invalid address: %d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
		return;
	}

	memset(&NewMachine, 0, sizeof(NewMachine));
	sprintf(NewMachine.m_szIpAddress, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
	NewMachine.m_nValidMachineIndex = NumMachines;
	NewMachine.m_nScanFlags = 1;

	// Make sure it doesn't already exist	
	if(FindIpAddress(ip) == (-1))
	{
		fseek(ValidMap, 0, SEEK_END);
		fwrite((void *)&NewMachine, 1, sizeof(TMapMachine), ValidMap);
	}
}

void AddRange(char *start, char *end)
{
	Tuint ipStart[4];
	Tuint ipEnd[4];
	Tuint ip[4];
	
	sscanf(start, "%d.%d.%d.%d", &ipStart[0], &ipStart[1], &ipStart[2], &ipStart[3]);
	sscanf(end, "%d.%d.%d.%d", &ipEnd[0], &ipEnd[1], &ipEnd[2], &ipEnd[3]);

	ip[0] = ipStart[0]; ip[1] = ipStart[1]; ip[2] = ipStart[2]; ip[3] = ipStart[3];
	for(ip[0] = ipStart[0]; ip[0] <= ipEnd[0]; ip[0]++)
	{
		if(ip[0] > 255)
			break;
		for(ip[1]=ipStart[1]; ip[1] <= ipEnd[1]; ip[1]++)
		{
			if(ip[1] > 255)
				break;

			for(ip[2]=ipStart[2]; ip[2] <= ipEnd[2]; ip[2]++)
			{
				if(ip[2] > 255)
					break;
				for(ip[3]=ipStart[3]; ip[3] <= ipEnd[3]; ip[3]++)
				{
					if(ip[3] > 355)
						break;
					
					printf("adding %d.%d.%d.%d\n", ip[0], ip[1], ip[2], ip[3]);
					AddIpAddress(ip);
				}							
			}			
		}
	}

}

void PrintUsage()
{
	printf("MapMaker -- ValidMachineMap creation utility\n");
	printf("\n");

	printf("\
usage:\n\
\n\
  mapmaker -init\n\
    Creates a new, empty ValidMachineMap\n\
\n\
  mapmaker -ip <ipaddress>\n\
    Adds a specific IP address to the Map\n\
\n\
  mapmaker -range <low-ip-address> <high-ip-address>\n\
    Adds a range of IP addresses to the Map\n\
    ex: mapmaker -range 129.186.0.0 129.186.255.255\n\
\n\
  mapmaker -v \n\
    View all IPs currently in the file\n\
\n\
  mapmaker -scan <threadnum>\n\
    Scan NetBios and Domain Names for all addresses in file\n"
	);
}

void main(int argc, char **args)
{
	bool handled=false;

	if(argc > 1)
	{
		argc--;
		args++;

		if(stricmp(*args, "-scan") == 0 && (argc == 2 || argc == 3))
		{
			args++;
			printf("scan current ips\n");
			ScanMachines(atoi(*args));
			return;
		}
		if(stricmp(*args, "-init") == 0)
		{
			printf("initializing Valid machine map\n");
			ValidMap = fopen("ValidMachineMap.dat", "wb");
			fclose(ValidMap);
			return;
		}
		
		ValidMap = fopen("ValidMachineMap.dat", "rb+");
		if(!ValidMap)
		{
			printf("Couldn't access ValidMachineMap.dat!");
			return;
		}
		InitMapMaker();
		
		if(stricmp(*args, "-ip") == 0 && argc == 2)
		{
			args++;
			printf("adding individual ip: %s\n", *args);			
			Tuint ip[4];
			sscanf(*args, "%d.%d.%d.%d", &ip[0], &ip[1], &ip[2], &ip[3]);
			AddIpAddress(ip);
			handled = true;
		}
		if(stricmp(*args, "-range") == 0 && argc == 3)
		{
			args++;
			printf("adding range: %s to %s\n", *args, *(args+1));
			AddRange(*args, *(args+1));
			handled = true;
		}
		if(stricmp(*args, "-v") == 0)
		{
			printf("view current ips\n");
			PrintAddresses();
			handled = true;
		}
		fclose(ValidMap);
	}
	if(!handled)
	{
		PrintUsage();
		return;
	}
}